@@ -39,8 +39,7 @@ source $NETWORKING_ODL_DIR/devstack/entry_points
 # Restore xtrace
 $_XTRACE_NETWORKING_ODL
 
-if [[ "$ODL_USING_EXISTING_JAVA" == "True" ]]
-then
+if [[ "$ODL_USING_EXISTING_JAVA" == "True" ]]; then
     echo 'Using installed java.'
     java -version || exit 1
 fi
@@ -82,6 +81,7 @@ if is_service_enabled odl-compute; then
             create_nova_conf_neutron
         fi
         bind_opendaylight_controller
+        sudo ovs-vsctl --may-exist add-br $OVS_BR
         wait_for_active_bridge $OVS_BR $ODL_RETRY_SLEEP_INTERVAL $ODL_BOOT_WAIT
 
         # L3 needs to be configured only for netvirt-ovsdb - in netvirt-vpnservice L3 is configured
