@@ -20,8 +20,11 @@ from oslo_log import log as logging
 
 from networking_bgpvpn.neutron.extensions import bgpvpn as bgpvpn_ext
 from networking_bgpvpn.neutron.services.service_drivers import driver_api
-from networking_odl._i18n import _LI
+from neutron_lib.api.definitions import bgpvpn as bgpvpn_const
+
 from networking_odl.common import constants as odl_const
+from networking_odl.common import postcommit
+from networking_odl.journal import full_sync
 from networking_odl.journal import journal
 
 
@@ -29,7 +32,17 @@ cfg.CONF.import_group('ml2_odl', 'networking_odl.common.config')
 
 LOG = logging.getLogger(__name__)
 
+BGPVPN_RESOURCES = {
+    odl_const.ODL_BGPVPN: odl_const.ODL_BGPVPNS,
+    odl_const.ODL_BGPVPN_NETWORK_ASSOCIATION:
+        odl_const.ODL_BGPVPN_NETWORK_ASSOCIATIONS,
 
+    odl_const.ODL_BGPVPN_ROUTER_ASSOCIATION:
+        odl_const.ODL_BGPVPN_ROUTER_ASSOCIATIONS
+}
+
+
+@postcommit.add_postcommit('bgpvpn', 'net_assoc', 'router_assoc')
 class OpenDaylightBgpvpnDriver(driver_api.BGPVPNDriver):
 
     """OpenDaylight BGPVPN Driver
@@ -40,24 +53,25 @@ class OpenDaylightBgpvpnDriver(driver_api.BGPVPNDriver):
 
     @log_helpers.log_method_call
     def __init__(self, service_plugin):
-        LOG.info(_LI("Initializing OpenDaylight BGPVPN v2 driver"))
+        LOG.info("Initializing OpenDaylight BGPVPN v2 driver")
         super(OpenDaylightBgpvpnDriver, self).__init__(service_plugin)
-        self.journal = journal.OpendaylightJournalThread()
+        self.journal = journal.OpenDaylightJournalThread()
+        full_sync.register(bgpvpn_const.LABEL, BGPVPN_RESOURCES)
 
     @log_helpers.log_method_call
     def create_bgpvpn_precommit(self, context, bgpvpn):
-        journal.record(context, None, odl_const.ODL_BGPVPN,
+        journal.record(context, odl_const.ODL_BGPVPN,
                        bgpvpn['id'], odl_const.ODL_CREATE, bgpvpn)
 
     @log_helpers.log_method_call
     def update_bgpvpn_precommit(self, context, bgpvpn):
-        journal.record(context, None, odl_const.ODL_BGPVPN,
+        journal.record(context, odl_const.ODL_BGPVPN,
                        bgpvpn['id'], odl_const.ODL_UPDATE, bgpvpn)
 
-    # TODO(yamahata): when bgpvpn driver interface implements precommit
-    # use this method.
+    @log_helpers.log_method_call
     def delete_bgpvpn_precommit(self, context, bgpvpn):
-        pass
+        journal.record(context, odl_const.ODL_BGPVPN,
+                       bgpvpn['id'], odl_const.ODL_DELETE, [])
 
     @log_helpers.log_method_call
     def create_net_assoc_precommit(self, context, net_assoc):
@@ -74,13 +88,14 @@ class OpenDaylightBgpvpnDriver(driver_api.BGPVPNDriver):
                         driver="OpenDaylight V2",
                         network=net_assoc['network_id'],
                         bgpvpn=bgpvpn['id'])
-        journal.record(context, None, odl_const.ODL_BGPVPN,
+        journal.record(context, odl_const.ODL_BGPVPN,
                        our_bgpvpn['id'], odl_const.ODL_UPDATE, our_bgpvpn)
 
-    # TODO(yamahata): when bgpvpn driver interface implements precommit
-    # use this method.
+    @log_helpers.log_method_call
     def delete_net_assoc_precommit(self, context, net_assoc):
-        pass
+        bgpvpn = self.get_bgpvpn(context, net_assoc['bgpvpn_id'])
+        journal.record(context, odl_const.ODL_BGPVPN,
+                       bgpvpn['id'], odl_const.ODL_UPDATE, bgpvpn)
 
     @log_helpers.log_method_call
     def create_router_assoc_precommit(self, context, router_assoc):
@@ -91,61 +106,11 @@ class OpenDaylightBgpvpnDriver(driver_api.BGPVPNDriver):
                 raise bgpvpn_ext.BGPVPNMultipleRouterAssocNotSupported(
                     driver="OpenDaylight V2")
         bgpvpn = self.get_bgpvpn(context, router_assoc['bgpvpn_id'])
-        journal.record(context, None, odl_const.ODL_BGPVPN,
+        journal.record(context, odl_const.ODL_BGPVPN,
                        bgpvpn['id'], odl_const.ODL_UPDATE, bgpvpn)
 
-    # TODO(yamahata): when bgpvpn driver interface implements precommit
-    # use this method.
+    @log_helpers.log_method_call
     def delete_router_assoc_precommit(self, context, router_assoc):
-        pass
-
-    def _postcommit(self):
-        self.journal.set_sync_event()
-
-    @log_helpers.log_method_call
-    def create_bgpvpn_postcommit(self, context, bgpvpn):
-        self._postcommit()
-
-    @log_helpers.log_method_call
-    def delete_bgpvpn_postcommit(self, context, bgpvpn):
-        # TODO(vivekanandan): The journal writing to be moved
-        # to _precommit, after enabling bgpvpn plugin in
-        # networking-bgpvpn to invoke _precommit for
-        # delete_bgpvpn lifecycle
-        journal.record(context, None, odl_const.ODL_BGPVPN,
-                       bgpvpn['id'], odl_const.ODL_DELETE, [])
-        self._postcommit()
-
-    @log_helpers.log_method_call
-    def update_bgpvpn_postcommit(self, context, old_bgpvpn, bgpvpn):
-        self._postcommit()
-
-    @log_helpers.log_method_call
-    def create_net_assoc_postcommit(self, context, net_assoc):
-        self._postcommit()
-
-    @log_helpers.log_method_call
-    def delete_net_assoc_postcommit(self, context, net_assoc):
-        # TODO(vivekanandan): The journal writing to be moved
-        # to _precommit, after enabling bgpvpn plugin in
-        # networking-bgpvpn to invoke _precommit for
-        # delete_net_assoc lifecycle
-        bgpvpn = self.get_bgpvpn(context, net_assoc['bgpvpn_id'])
-        journal.record(context, None, odl_const.ODL_BGPVPN,
-                       bgpvpn['id'], odl_const.ODL_UPDATE, bgpvpn)
-        self._postcommit()
-
-    @log_helpers.log_method_call
-    def create_router_assoc_postcommit(self, context, router_assoc):
-        self._postcommit()
-
-    @log_helpers.log_method_call
-    def delete_router_assoc_postcommit(self, context, router_assoc):
-        # TODO(vivekanandan): The journal writing to be moved
-        # to _precommit, after enabling bgpvpn plugin in
-        # networking-bgpvpn to invoke _precommit for
-        # delete_router_assoc lifecycle
         bgpvpn = self.get_bgpvpn(context, router_assoc['bgpvpn_id'])
-        journal.record(context, None, odl_const.ODL_BGPVPN,
+        journal.record(context, odl_const.ODL_BGPVPN,
                        bgpvpn['id'], odl_const.ODL_UPDATE, bgpvpn)
-        self._postcommit()
