@@ -39,22 +39,43 @@ ODL_MEMBERS = 'members'
 ODL_HEALTHMONITOR = 'healthmonitor'
 ODL_HEALTHMONITORS = 'healthmonitors'
 
+ODL_QOS = 'qos'
 ODL_QOS_POLICY = 'policy'
-ODL_QOS_POLICIES = 'qos/policies'
+ODL_QOS_POLICIES = 'policies'
 
 ODL_SFC = 'sfc'
 ODL_SFC_FLOW_CLASSIFIER = 'flowclassifier'
+ODL_SFC_FLOW_CLASSIFIERS = 'flowclassifiers'
 ODL_SFC_PORT_PAIR = 'portpair'
+ODL_SFC_PORT_PAIRS = 'portpairs'
 ODL_SFC_PORT_PAIR_GROUP = 'portpairgroup'
+ODL_SFC_PORT_PAIR_GROUPS = 'portpairgroups'
 ODL_SFC_PORT_CHAIN = 'portchain'
+ODL_SFC_PORT_CHAINS = 'portchains'
+
+NETWORKING_SFC_FLOW_CLASSIFIER = 'flow_classifier'
+NETWORKING_SFC_FLOW_CLASSIFIERS = 'flow_classifiers'
+NETWORKING_SFC_PORT_PAIR = 'port_pair'
+NETWORKING_SFC_PORT_PAIRS = 'port_pairs'
+NETWORKING_SFC_PORT_PAIR_GROUP = 'port_pair_group'
+NETWORKING_SFC_PORT_PAIR_GROUPS = 'port_pair_groups'
+NETWORKING_SFC_PORT_CHAIN = 'port_chain'
+NETWORKING_SFC_PORT_CHAINS = 'port_chains'
 
 ODL_TRUNK = 'trunk'
 ODL_TRUNKS = 'trunks'
 
 ODL_L2GATEWAY = 'l2_gateway'
+ODL_L2GATEWAYS = 'l2_gateways'
 ODL_L2GATEWAY_CONNECTION = 'l2gateway_connection'
+ODL_L2GATEWAY_CONNECTIONS = 'l2_gateway_connections'
 
 ODL_BGPVPN = 'bgpvpn'
+ODL_BGPVPNS = 'bgpvpns'
+ODL_BGPVPN_NETWORK_ASSOCIATION = 'bgpvpn_network_association'
+ODL_BGPVPN_NETWORK_ASSOCIATIONS = 'bgpvpn_network_associations'
+ODL_BGPVPN_ROUTER_ASSOCIATION = 'bgpvpn_network_association'
+ODL_BGPVPN_ROUTER_ASSOCIATIONS = 'bgpvpn_network_associations'
 
 ODL_ML2_MECH_DRIVER_V1 = "opendaylight"
 ODL_ML2_MECH_DRIVER_V2 = "opendaylight_v2"
@@ -63,20 +84,14 @@ ODL_CREATE = 'create'
 ODL_UPDATE = 'update'
 ODL_DELETE = 'delete'
 
-L2_RESOURCES = {ODL_SG: ODL_SGS,
-                ODL_SG_RULE: ODL_SG_RULES,
-                ODL_NETWORK: ODL_NETWORKS,
-                ODL_SUBNET: ODL_SUBNETS,
-                ODL_PORT: ODL_PORTS}
-L3_RESOURCES = {ODL_ROUTER: ODL_ROUTERS,
-                ODL_FLOATINGIP: ODL_FLOATINGIPS}
-
-
 # Constants for journal operation states
 PENDING = 'pending'
 PROCESSING = 'processing'
 FAILED = 'failed'
 COMPLETED = 'completed'
 
-# dict to store url prefixes
-PREFIXES = {ODL_QOS_POLICY: 'qos'}
+# Journal Callback events
+BEFORE_COMPLETE = 'before_complete'
+
+# dict to store url mappings
+RESOURCE_URL_MAPPINGS = {ODL_QOS_POLICY: "%s/%s" % (ODL_QOS, ODL_QOS_POLICIES)}
