@@ -23,14 +23,20 @@ from neutron.db import l3_agentschedulers_db
 from neutron.db import l3_dvr_db
 from neutron.db import l3_gwmode_db
 from neutron_lib import constants as q_const
+from neutron_lib.plugins import constants as plugin_constants
 
 from networking_odl.common import config  # noqa
 from networking_odl.common import constants as odl_const
-from networking_odl.db import db
+from networking_odl.journal import full_sync
 from networking_odl.journal import journal
 
 LOG = logging.getLogger(__name__)
 
+L3_RESOURCES = {
+    odl_const.ODL_ROUTER: odl_const.ODL_ROUTERS,
+    odl_const.ODL_FLOATINGIP: odl_const.ODL_FLOATINGIPS
+}
+
 
 class OpenDaylightL3RouterPlugin(
         common_db_mixin.CommonDbMixin,
@@ -53,10 +59,11 @@ class OpenDaylightL3RouterPlugin(
 
         # TODO(rcurran): Continue investigation into how many journal threads
         # to run per neutron controller deployment.
-        self.journal = journal.OpendaylightJournalThread()
+        self.journal = journal.OpenDaylightJournalThread()
+        full_sync.register(plugin_constants.L3, L3_RESOURCES)
 
     def get_plugin_type(self):
-        return q_const.L3
+        return plugin_constants.L3
 
     def get_plugin_description(self):
         """Returns string description of the plugin."""
@@ -65,55 +72,55 @@ class OpenDaylightL3RouterPlugin(
 
     @journal.call_thread_on_end
     def create_router(self, context, router):
-        session = db_api.get_session()
+        session = db_api.get_writer_session()
         with session.begin(subtransactions=True):
             router_dict = super(
                 OpenDaylightL3RouterPlugin, self).create_router(context,
                                                                 router)
-            db.create_pending_row(context.session, odl_const.ODL_ROUTER,
-                                  router_dict['id'], odl_const.ODL_CREATE,
-                                  router_dict)
+            journal.record(context, odl_const.ODL_ROUTER, router_dict['id'],
+                           odl_const.ODL_CREATE, router_dict)
         return router_dict
 
     @journal.call_thread_on_end
     def update_router(self, context, router_id, router):
-        session = db_api.get_session()
+        session = db_api.get_writer_session()
         with session.begin(subtransactions=True):
             router_dict = super(
                 OpenDaylightL3RouterPlugin, self).update_router(
                     context, router_id, router)
-            db.create_pending_row(context.session, odl_const.ODL_ROUTER,
-                                  router_id, odl_const.ODL_UPDATE, router_dict)
+            journal.record(context, odl_const.ODL_ROUTER,
+                           router_id, odl_const.ODL_UPDATE, router_dict)
         return router_dict
 
     @journal.call_thread_on_end
     def delete_router(self, context, router_id):
-        session = db_api.get_session()
+        session = db_api.get_writer_session()
         router_dict = self.get_router(context, router_id)
         dependency_list = [router_dict['gw_port_id']]
         with session.begin(subtransactions=True):
             super(OpenDaylightL3RouterPlugin, self).delete_router(context,
                                                                   router_id)
-            db.create_pending_row(context.session, odl_const.ODL_ROUTER,
-                                  router_id, odl_const.ODL_DELETE,
-                                  dependency_list)
+            journal.record(context, odl_const.ODL_ROUTER, router_id,
+                           odl_const.ODL_DELETE, dependency_list)
 
     @journal.call_thread_on_end
     def create_floatingip(self, context, floatingip,
                           initial_status=q_const.FLOATINGIP_STATUS_ACTIVE):
-        session = db_api.get_session()
+        fip = floatingip['floatingip']
+        if fip.get('port_id') is None:
+            initial_status = q_const.FLOATINGIP_STATUS_DOWN
+        session = db_api.get_writer_session()
         with session.begin(subtransactions=True):
             fip_dict = super(
                 OpenDaylightL3RouterPlugin, self).create_floatingip(
                     context, floatingip, initial_status)
-            db.create_pending_row(context.session, odl_const.ODL_FLOATINGIP,
-                                  fip_dict['id'], odl_const.ODL_CREATE,
-                                  fip_dict)
+            journal.record(context, odl_const.ODL_FLOATINGIP, fip_dict['id'],
+                           odl_const.ODL_CREATE, fip_dict)
         return fip_dict
 
     @journal.call_thread_on_end
     def update_floatingip(self, context, floatingip_id, floatingip):
-        session = db_api.get_session()
+        session = db_api.get_writer_session()
         with session.begin(subtransactions=True):
             fip_dict = super(
                 OpenDaylightL3RouterPlugin, self).update_floatingip(
@@ -127,27 +134,25 @@ class OpenDaylightL3RouterPlugin(
             self.update_floatingip_status(context, floatingip_id,
                                           fip_dict['status'])
 
-            db.create_pending_row(context.session, odl_const.ODL_FLOATINGIP,
-                                  floatingip_id, odl_const.ODL_UPDATE,
-                                  fip_dict)
+            journal.record(context, odl_const.ODL_FLOATINGIP, floatingip_id,
+                           odl_const.ODL_UPDATE, fip_dict)
         return fip_dict
 
     @journal.call_thread_on_end
     def delete_floatingip(self, context, floatingip_id):
-        session = db_api.get_session()
+        session = db_api.get_writer_session()
         floatingip_dict = self.get_floatingip(context, floatingip_id)
         dependency_list = [floatingip_dict['router_id']]
         dependency_list.append(floatingip_dict['floating_network_id'])
         with session.begin(subtransactions=True):
             super(OpenDaylightL3RouterPlugin, self).delete_floatingip(
                 context, floatingip_id)
-            db.create_pending_row(context.session, odl_const.ODL_FLOATINGIP,
-                                  floatingip_id, odl_const.ODL_DELETE,
-                                  dependency_list)
+            journal.record(context, odl_const.ODL_FLOATINGIP, floatingip_id,
+                           odl_const.ODL_DELETE, dependency_list)
 
     @journal.call_thread_on_end
     def add_router_interface(self, context, router_id, interface_info):
-        session = db_api.get_session()
+        session = db_api.get_writer_session()
         with session.begin(subtransactions=True):
             new_router = super(
                 OpenDaylightL3RouterPlugin, self).add_router_interface(
@@ -156,7 +161,7 @@ class OpenDaylightL3RouterPlugin(
 
     @journal.call_thread_on_end
     def remove_router_interface(self, context, router_id, interface_info):
-        session = db_api.get_session()
+        session = db_api.get_writer_session()
         with session.begin(subtransactions=True):
             new_router = super(
                 OpenDaylightL3RouterPlugin, self).remove_router_interface(
