@@ -16,10 +16,9 @@
 
 from oslo_log import log
 
-from neutron.plugins.common import constants
-from neutron.plugins.ml2 import driver_api
 from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants as n_const
+from neutron_lib.plugins.ml2 import api
 
 from networking_odl.ml2 import port_binding
 
@@ -59,7 +58,7 @@ class LegacyPortBindingManager(port_binding.PortBindingController):
                        'segment': valid_segment, 'vif_type': vif_type})
 
             port_context.set_binding(
-                segment[driver_api.ID], vif_type,
+                valid_segment[api.ID], vif_type,
                 self.vif_details,
                 status=n_const.PORT_STATUS_ACTIVE)
 
@@ -70,10 +69,10 @@ class LegacyPortBindingManager(port_binding.PortBindingController):
         False to indicate this to callers.
         """
 
-        network_type = segment[driver_api.NETWORK_TYPE]
-        return network_type in [constants.TYPE_FLAT, constants.TYPE_LOCAL,
-                                constants.TYPE_GRE, constants.TYPE_VXLAN,
-                                constants.TYPE_VLAN]
+        network_type = segment[api.NETWORK_TYPE]
+        return network_type in [n_const.TYPE_FLAT, n_const.TYPE_LOCAL,
+                                n_const.TYPE_GRE, n_const.TYPE_VXLAN,
+                                n_const.TYPE_VLAN]
 
     def _get_vif_type(self, port_context):
         """Get VIF type string for given PortContext
