@@ -15,33 +15,35 @@
 
 import abc
 import copy
+
 import six
 
 import netaddr
+from neutron_lib.api.definitions import provider_net as providernet
+from neutron_lib.callbacks import resources
+from neutron_lib import constants as p_const
+from neutron_lib import context as neutron_context
+from neutron_lib import exceptions as n_exc
+from neutron_lib.plugins.ml2 import api
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
 import requests
 
-from neutron.callbacks import resources
 from neutron.common import utils
-from neutron import context as neutron_context
 from neutron.extensions import allowedaddresspairs as addr_pair
 from neutron.extensions import multiprovidernet as mpnet
 from neutron.extensions import securitygroup as sg
-from neutron.plugins.common import constants as p_const
-from neutron.plugins.ml2 import driver_api
 from neutron.plugins.ml2 import driver_context
-from neutron_lib.api.definitions import provider_net as providernet
-from neutron_lib import exceptions as n_exc
 
-from networking_odl._i18n import _LE
 from networking_odl.common import callback as odl_call
 from networking_odl.common import client as odl_client
 from networking_odl.common import constants as odl_const
 from networking_odl.common import filters
+from networking_odl.common import odl_features
 from networking_odl.common import utils as odl_utils
 from networking_odl.ml2 import port_binding
+from networking_odl.ml2 import port_status_update
 from networking_odl.trunk import trunk_driver_v1 as trunk_driver
 
 
@@ -197,10 +199,9 @@ class PortFilter(ResourceFilterBase):
     @classmethod
     def filter_create_attributes_with_plugin(cls, port, plugin, dbcontext):
         network = plugin.get_network(dbcontext, port['network_id'])
-        # TODO(yamahata): port binding
-        binding = {}
+        port_db = plugin._get_port(dbcontext, port['id'])
         context = driver_context.PortContext(
-            plugin, dbcontext, port, network, binding, None)
+            plugin, dbcontext, port, network, port_db.port_binding, None)
         cls.filter_create_attributes(port, context)
 
 
@@ -373,8 +374,8 @@ class OpenDaylightDriver(object):
                                      {object_type_url[:-1]: resource})
         except Exception:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Unable to perform %(operation)s on "
-                              "%(object_type)s %(object_id)s"),
+                LOG.error("Unable to perform %(operation)s on "
+                          "%(object_type)s %(object_id)s",
                           {'operation': operation,
                            'object_type': object_type,
                            'object_id': obj_id})
@@ -397,9 +398,8 @@ class OpenDaylightDriver(object):
                 self.client.sendjson(method, urlpath, resource_dict)
         except Exception:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Unable to perform %(operation)s on "
-                              "%(object_type)s %(res_id)s "
-                              "%(resource_dict)s"),
+                LOG.error("Unable to perform %(operation)s on "
+                          "%(object_type)s %(res_id)s %(resource_dict)s",
                           {'operation': operation,
                            'object_type': object_type,
                            'res_id': res_id,
@@ -446,7 +446,7 @@ class OpenDaylightDriver(object):
                    for segment in segments)
 
 
-class OpenDaylightMechanismDriver(driver_api.MechanismDriver):
+class OpenDaylightMechanismDriver(api.MechanismDriver):
 
     """Mechanism Driver for OpenDaylight.
 
@@ -462,6 +462,12 @@ class OpenDaylightMechanismDriver(driver_api.MechanismDriver):
         self.password = cfg.CONF.ml2_odl.password
         self.odl_drv = OpenDaylightDriver()
         self.trunk_driver = trunk_driver.OpenDaylightTrunkDriverV1.create()
+        odl_features.init()
+
+    def get_workers(self):
+        workers = [port_status_update.OdlPortStatusUpdate()]
+        workers += self.odl_drv.port_binding_controller.get_workers()
+        return workers
 
     # Postcommit hooks are used to trigger synchronization.
 
