@@ -14,30 +14,43 @@
 #    under the License.
 
 from oslo_config import cfg
+from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
 
-from neutron.db.models import securitygroup
 from neutron.extensions import multiprovidernet as mpnet
-from neutron.plugins.common import constants as p_const
-from neutron.plugins.ml2 import driver_api as api
 from neutron_lib.api.definitions import provider_net as providernet
-from neutron_lib.plugins import directory
+from neutron_lib import constants as p_const
+from neutron_lib.plugins import constants as nlib_const
+from neutron_lib.plugins.ml2 import api
 
-from networking_odl._i18n import _, _LE
 from networking_odl.common import callback
 from networking_odl.common import config as odl_conf
 from networking_odl.common import constants as odl_const
+from networking_odl.common import odl_features
+from networking_odl.common import postcommit
+from networking_odl.dhcp import odl_dhcp_driver as dhcp_driver
 from networking_odl.journal import cleanup
 from networking_odl.journal import full_sync
 from networking_odl.journal import journal
-from networking_odl.journal import maintenance
+from networking_odl.journal import periodic_task
 from networking_odl.journal import recovery
 from networking_odl.ml2 import port_binding
+from networking_odl.ml2 import port_status_update
+from networking_odl.qos import qos_driver_v2 as qos_driver
 from networking_odl.trunk import trunk_driver_v2 as trunk_driver
 
 LOG = logging.getLogger(__name__)
 
+L2_RESOURCES = {
+    odl_const.ODL_SG: odl_const.ODL_SGS,
+    odl_const.ODL_SG_RULE: odl_const.ODL_SG_RULES,
+    odl_const.ODL_NETWORK: odl_const.ODL_NETWORKS,
+    odl_const.ODL_SUBNET: odl_const.ODL_SUBNETS,
+    odl_const.ODL_PORT: odl_const.ODL_PORTS
+}
 
+
+@postcommit.add_postcommit('network', 'subnet', 'port')
 class OpenDaylightMechanismDriver(api.MechanismDriver):
     """OpenDaylight Python Driver for Neutron.
 
@@ -51,62 +64,86 @@ class OpenDaylightMechanismDriver(api.MechanismDriver):
         self.sg_handler = callback.OdlSecurityGroupsHandler(
             self.sync_from_callback_precommit,
             self.sync_from_callback_postcommit)
-        self.journal = journal.OpendaylightJournalThread()
+        self.journal = journal.OpenDaylightJournalThread()
         self.port_binding_controller = port_binding.PortBindingManager.create()
         self.trunk_driver = trunk_driver.OpenDaylightTrunkDriverV2.create()
-        self._start_maintenance_thread()
+        if odl_const.ODL_QOS in cfg.CONF.ml2.extension_drivers:
+            qos_driver.OpenDaylightQosDriver.create()
+        if cfg.CONF.ml2_odl.enable_dhcp_service:
+            self.dhcp_driver = dhcp_driver.OdlDhcpDriver()
 
-    def _start_maintenance_thread(self):
-        # start the maintenance thread and register all the maintenance
+        self._start_periodic_task()
+        full_sync.register(nlib_const.CORE, L2_RESOURCES)
+        odl_features.init()
+
+    def get_workers(self):
+        workers = [port_status_update.OdlPortStatusUpdate(),
+                   journal.JournalPeriodicProcessor()]
+        workers += self.port_binding_controller.get_workers()
+        return workers
+
+    def _start_periodic_task(self):
+        # start the periodic task and register all the phases
         # operations :
         # (1) JournalCleanup - Delete completed rows from journal
         # (2) CleanupProcessing - Mark orphaned processing rows to pending
         # (3) Full sync - Re-sync when detecting an ODL "cold reboot"
         cleanup_obj = cleanup.JournalCleanup()
-        self._maintenance_thread = maintenance.MaintenanceThread()
-        self._maintenance_thread.register_operation(
+        interval = cfg.CONF.ml2_odl.restconf_poll_interval
+        self._periodic_task = periodic_task.PeriodicTask('maintenance',
+                                                         interval)
+        self._periodic_task.register_operation(
             cleanup_obj.delete_completed_rows)
-        self._maintenance_thread.register_operation(
+        self._periodic_task.register_operation(
             cleanup_obj.cleanup_processing_rows)
-        self._maintenance_thread.register_operation(full_sync.full_sync)
-        self._maintenance_thread.register_operation(recovery.journal_recovery)
-        self._maintenance_thread.start()
+        self._periodic_task.register_operation(full_sync.full_sync)
+        self._periodic_task.register_operation(recovery.journal_recovery)
+        self._periodic_task.start()
 
     @staticmethod
     def _record_in_journal(context, object_type, operation, data=None):
         if data is None:
             data = context.current
-        journal.record(context._plugin_context, context, object_type,
-                       context.current['id'], operation, data)
+        journal.record(context._plugin_context, object_type,
+                       context.current['id'], operation, data,
+                       ml2_context=context)
 
+    @log_helpers.log_method_call
     def create_network_precommit(self, context):
         OpenDaylightMechanismDriver._record_in_journal(
             context, odl_const.ODL_NETWORK, odl_const.ODL_CREATE)
 
+    @log_helpers.log_method_call
     def create_subnet_precommit(self, context):
         OpenDaylightMechanismDriver._record_in_journal(
             context, odl_const.ODL_SUBNET, odl_const.ODL_CREATE)
 
+    @log_helpers.log_method_call
     def create_port_precommit(self, context):
         OpenDaylightMechanismDriver._record_in_journal(
             context, odl_const.ODL_PORT, odl_const.ODL_CREATE)
 
+    @log_helpers.log_method_call
     def update_network_precommit(self, context):
         OpenDaylightMechanismDriver._record_in_journal(
             context, odl_const.ODL_NETWORK, odl_const.ODL_UPDATE)
 
+    @log_helpers.log_method_call
     def update_subnet_precommit(self, context):
         OpenDaylightMechanismDriver._record_in_journal(
             context, odl_const.ODL_SUBNET, odl_const.ODL_UPDATE)
 
+    @log_helpers.log_method_call
     def update_port_precommit(self, context):
         OpenDaylightMechanismDriver._record_in_journal(
             context, odl_const.ODL_PORT, odl_const.ODL_UPDATE)
 
+    @log_helpers.log_method_call
     def delete_network_precommit(self, context):
         OpenDaylightMechanismDriver._record_in_journal(
             context, odl_const.ODL_NETWORK, odl_const.ODL_DELETE, data=[])
 
+    @log_helpers.log_method_call
     def delete_subnet_precommit(self, context):
         # Use the journal row's data field to store parent object
         # uuids. This information is required for validation checking
@@ -116,6 +153,7 @@ class OpenDaylightMechanismDriver(api.MechanismDriver):
             context, odl_const.ODL_SUBNET, odl_const.ODL_DELETE,
             data=new_context)
 
+    @log_helpers.log_method_call
     def delete_port_precommit(self, context):
         # Use the journal row's data field to store parent object
         # uuids. This information is required for validation checking
@@ -127,58 +165,18 @@ class OpenDaylightMechanismDriver(api.MechanismDriver):
             context, odl_const.ODL_PORT, odl_const.ODL_DELETE,
             data=new_context)
 
-    def _make_security_group_dict(self, sg):
-        return {
-            'id': sg['id'],
-            'name': sg['name'],
-            'tenant_id': sg['tenant_id'],
-            'description': sg['description']
-        }
-
-    def _make_security_group_rule_dict(self, rule, sg_id=None):
-        if sg_id is None:
-            sg_id = rule['security_group_id']
-        return {
-            'id': rule['id'],
-            'tenant_id': rule['tenant_id'],
-            'security_group_id': sg_id,
-            'ethertype': rule['ethertype'],
-            'direction': rule['direction'],
-            'protocol': rule['protocol'],
-            'port_range_min': rule['port_range_min'],
-            'port_range_max': rule['port_range_max'],
-            'remote_ip_prefix': rule['remote_ip_prefix'],
-            'remote_group_id': rule['remote_group_id']
-        }
-
     def _sync_security_group_create_precommit(
-            self, context, operation, object_type, res_id, resource_dict):
-        # TODO(yamahata): remove this work around once
-        # https://review.openstack.org/#/c/281693/
-        # is merged.
-        # For now, SG rules aren't passed down with
-        # precommit event. We resort to get it by query.
-        new_objects = context.session.new
-        sgs = [sg for sg in new_objects
-               if isinstance(sg, securitygroup.SecurityGroup)]
-        if res_id is not None:
-            sgs = [sg for sg in sgs if sg.id == res_id]
-        for sg in sgs:
-            sg_id = sg['id']
-            res = self._make_security_group_dict(sg)
-            journal.record(context, None, object_type, sg_id, operation, res)
-            # NOTE(yamahata): when security group is created, default rules
-            # are also created.
-            # NOTE(yamahata): at this point, rule.security_group_id isn't
-            # populated. but it has rule.security_group
-            rules = [rule for rule in new_objects
-                     if (isinstance(rule, securitygroup.SecurityGroupRule) and
-                         rule.security_group == sg)]
-            for rule in rules:
-                res_rule = self._make_security_group_rule_dict(rule, sg_id)
-                journal.record(context, None, odl_const.ODL_SG_RULE,
-                               rule['id'], odl_const.ODL_CREATE, res_rule)
+            self, context, operation, object_type, res_id, sg_dict):
 
+        journal.record(context, object_type, sg_dict['id'], operation, sg_dict)
+
+        # NOTE(yamahata): when security group is created, default rules
+        # are also created.
+        for rule in sg_dict['security_group_rules']:
+            journal.record(context, odl_const.ODL_SG_RULE, rule['id'],
+                           odl_const.ODL_CREATE, rule)
+
+    @log_helpers.log_method_call
     def sync_from_callback_precommit(self, context, operation, res_type,
                                      res_id, resource_dict, **kwargs):
         object_type = res_type.singular
@@ -191,63 +189,22 @@ class OpenDaylightMechanismDriver(api.MechanismDriver):
                 context, operation, object_type, res_id, resource_dict)
             return
 
-        # NOTE(yamahata): in security group/security gorup rule case,
-        # orm object is passed. not resource dict. So we have to convert it
-        # into resource_dict
-        if not isinstance(resource_dict, dict) and resource_dict is not None:
-            if object_type == odl_const.ODL_SG:
-                resource_dict = self._make_security_group_dict(resource_dict)
-            elif object_type == odl_const.ODL_SG_RULE:
-                resource_dict = self._make_security_group_rule_dict(
-                    resource_dict)
-        # NOTE(yamahata): bug work around
-        # callback for update of security grouop doesn't pass complete
-        # info. So we have to build it. Once the bug is fixed, remove
-        # this bug work around.
-        # https://launchpad.net/bugs/1546910
-        # https://review.openstack.org/#/c/281693/
-        elif (object_type == odl_const.ODL_SG and
-              operation == odl_const.ODL_UPDATE):
-            # NOTE(yamahata): precommit_update is called before updating
-            # values. so context.session.{new, dirty} doesn't include sg
-            # in question. a dictionary with new values needs to be build.
-            core_plugin = directory.get_plugin()
-            sg = core_plugin._get_security_group(context, res_id)
-            tmp_dict = self._make_security_group_dict(sg)
-            tmp_dict.update(resource_dict)
-            resource_dict = tmp_dict
-
         object_uuid = (resource_dict.get('id')
                        if operation == 'create' else res_id)
-        if object_uuid is None:
-            # NOTE(yamahata): bug work around bug/1546910
-            # TODO(yamahata): once the following patch is merged
-            # remove this bug work around
-            # https://review.openstack.org/#/c/281693/
-            assert object_type == odl_const.ODL_SG_RULE
-            # NOTE(yamahata): bulk creation case
-            # context.session.new accumulates all newly created orm object.
-            # there is no easy way to pick up the lastly added orm object.
-            rules = [rule for rule in context.session.new
-                     if (isinstance(rule, securitygroup.SecurityGroupRule))]
-            if len(rules) == 1:
-                object_uuid = rules[0].id
-                resource_dict['id'] = object_uuid
-            else:
-                LOG.error(_LE("bulk creation of sgrule isn't supported"))
-                raise NotImplementedError(
-                    _("unsupporetd bulk creation of security group rule"))
-        journal.record(context, None, object_type, object_uuid,
-                       operation, resource_dict)
+
         # NOTE(yamahata): DB auto deletion
         # Security Group Rule under this Security Group needs to
         # be deleted. At NeutronDB layer rules are auto deleted with
         # cascade='all,delete'.
         if (object_type == odl_const.ODL_SG and
                 operation == odl_const.ODL_DELETE):
-            for rule in kwargs['security_group'].rules:
-                journal.record(context, None, odl_const.ODL_SG_RULE,
-                               rule.id, odl_const.ODL_DELETE, [object_uuid])
+            for rule_id in kwargs['security_group_rule_ids']:
+                journal.record(context, odl_const.ODL_SG_RULE,
+                               rule_id, odl_const.ODL_DELETE, [object_uuid])
+
+        assert object_uuid is not None
+        journal.record(context, object_type, object_uuid,
+                       operation, resource_dict)
 
     def sync_from_callback_postcommit(self, context, operation, res_type,
                                       res_id, resource_dict, **kwargs):
@@ -256,16 +213,7 @@ class OpenDaylightMechanismDriver(api.MechanismDriver):
     def _postcommit(self, context):
         self.journal.set_sync_event()
 
-    create_network_postcommit = _postcommit
-    create_subnet_postcommit = _postcommit
-    create_port_postcommit = _postcommit
-    update_network_postcommit = _postcommit
-    update_subnet_postcommit = _postcommit
-    update_port_postcommit = _postcommit
-    delete_network_postcommit = _postcommit
-    delete_subnet_postcommit = _postcommit
-    delete_port_postcommit = _postcommit
-
+    @log_helpers.log_method_call
     def bind_port(self, port_context):
         """Set binding for a valid segments
 
