@@ -16,30 +16,17 @@
 
 import mock
 
-from neutron.db import api as neutron_db_api
-
 from networking_odl.bgpvpn import odl_v2 as driverv2
 from networking_odl.common import constants as odl_const
 from networking_odl.db import db
-from networking_odl.journal import journal
 from networking_odl.tests.unit import base_v2
 
 
-class OpendaylightBgpvpnDriverTestCase(base_v2.OpenDaylightConfigBase):
+class OpenDaylightBgpvpnDriverTestCase(base_v2.OpenDaylightConfigBase):
 
     def setUp(self):
-        super(OpendaylightBgpvpnDriverTestCase, self).setUp()
-        self.db_session = neutron_db_api.get_reader_session()
+        super(OpenDaylightBgpvpnDriverTestCase, self).setUp()
         self.driver = driverv2.OpenDaylightBgpvpnDriver(service_plugin=None)
-        self.context = self._get_mock_context()
-        self.mock_sync_thread = mock.patch.object(
-            journal.OpendaylightJournalThread, 'start_odl_sync_thread').start()
-        self.thread = journal.OpendaylightJournalThread()
-
-    def _get_mock_context(self):
-        context = mock.Mock()
-        context.session = self.db_session
-        return context
 
     def _get_fake_bgpvpn(self, net=False, router=False):
         net_id = []
@@ -48,12 +35,9 @@ class OpendaylightBgpvpnDriverTestCase(base_v2.OpenDaylightConfigBase):
             router_id = ['ROUTER_ID']
         if net:
             net_id = ['NET_ID']
-        fake_bgpvpn = {'export_targets': mock.ANY, 'name': mock.ANY,
-                       'route_targets': '100:1', 'tenant_id': mock.ANY,
-                       'project_id': mock.ANY,
-                       'import_targets': mock.ANY,
+        fake_bgpvpn = {'route_targets': '100:1',
                        'route_distinguishers': ['100:1'],
-                       'type': mock.ANY, 'id': 'BGPVPN_ID',
+                       'id': 'BGPVPN_ID',
                        'networks': net_id,
                        'routers': router_id}
         return fake_bgpvpn
@@ -75,6 +59,7 @@ class OpendaylightBgpvpnDriverTestCase(base_v2.OpenDaylightConfigBase):
                                                   odl_const.PENDING),
                       key=lambda x: x.seqnum)
         if precommit:
+            self.db_session.flush()
             self.assertEqual(operation, rows[0]['operation'])
             self.assertEqual(object_type, rows[0]['object_type'])
             self.assertEqual(data['id'], rows[0]['object_uuid'])
@@ -83,26 +68,28 @@ class OpendaylightBgpvpnDriverTestCase(base_v2.OpenDaylightConfigBase):
 
     def test_create_bgpvpn(self):
         fake_data = self._get_fake_bgpvpn()
-        self.driver.create_bgpvpn_precommit(self.context, fake_data)
+        self.driver.create_bgpvpn_precommit(self.db_context, fake_data)
         self._assert_op(odl_const.ODL_CREATE, odl_const.ODL_BGPVPN,
                         fake_data)
-        self.thread.run_sync_thread(exit_after_run=True)
+        self.run_journal_processing()
         self._assert_op(odl_const.ODL_CREATE, odl_const.ODL_BGPVPN,
                         fake_data, False)
 
     def test_update_bgpvpn(self):
         fake_data = self._get_fake_bgpvpn()
-        self.driver.update_bgpvpn_precommit(self.context, fake_data)
+        self.driver.update_bgpvpn_precommit(self.db_context, fake_data)
         self._assert_op(odl_const.ODL_UPDATE, odl_const.ODL_BGPVPN,
                         fake_data)
-        self.thread.run_sync_thread(exit_after_run=True)
+        self.run_journal_processing()
         self._assert_op(odl_const.ODL_UPDATE, odl_const.ODL_BGPVPN,
                         fake_data, False)
 
     def test_delete_bgpvpn(self):
         fake_data = self._get_fake_bgpvpn()
-        self.driver.delete_bgpvpn_postcommit(self.context, fake_data)
-        self.thread.run_sync_thread(exit_after_run=True)
+        self.driver.delete_bgpvpn_precommit(self.db_context, fake_data)
+        self._assert_op(odl_const.ODL_DELETE, odl_const.ODL_BGPVPN,
+                        fake_data)
+        self.run_journal_processing()
         self._assert_op(odl_const.ODL_DELETE, odl_const.ODL_BGPVPN,
                         fake_data, False)
 
@@ -113,12 +100,12 @@ class OpendaylightBgpvpnDriverTestCase(base_v2.OpenDaylightConfigBase):
                                return_value=[]), \
             mock.patch.object(self.driver, 'get_bgpvpn',
                               return_value=fake_rtr_upd_bgpvpn_data):
-            self.driver.create_router_assoc_precommit(self.context,
+            self.driver.create_router_assoc_precommit(self.db_context,
                                                       fake_rtr_assoc_data)
             self._assert_op(odl_const.ODL_UPDATE,
                             odl_const.ODL_BGPVPN,
                             fake_rtr_upd_bgpvpn_data)
-            self.thread.run_sync_thread(exit_after_run=True)
+            self.run_journal_processing()
             self._assert_op(odl_const.ODL_UPDATE,
                             odl_const.ODL_BGPVPN,
                             fake_rtr_upd_bgpvpn_data, False)
@@ -128,9 +115,12 @@ class OpendaylightBgpvpnDriverTestCase(base_v2.OpenDaylightConfigBase):
         fake_bgpvpn_data = self._get_fake_bgpvpn(router=False)
         with mock.patch.object(self.driver, 'get_bgpvpn',
                                return_value=fake_bgpvpn_data):
-            self.driver.delete_router_assoc_postcommit(self.context,
-                                                       fake_rtr_assoc_data)
-            self.thread.run_sync_thread(exit_after_run=True)
+            self.driver.delete_router_assoc_precommit(self.db_context,
+                                                      fake_rtr_assoc_data)
+            self._assert_op(odl_const.ODL_UPDATE,
+                            odl_const.ODL_BGPVPN,
+                            fake_bgpvpn_data)
+            self.run_journal_processing()
             self._assert_op(odl_const.ODL_UPDATE,
                             odl_const.ODL_BGPVPN,
                             fake_bgpvpn_data, False)
@@ -141,12 +131,12 @@ class OpendaylightBgpvpnDriverTestCase(base_v2.OpenDaylightConfigBase):
         # todo(vivekanandan) add check for case when assoc already exists
         with mock.patch.object(self.driver, 'get_bgpvpns',
                                return_value=[fake_net_upd_bgpvpn_data]):
-            self.driver.create_net_assoc_precommit(self.context,
+            self.driver.create_net_assoc_precommit(self.db_context,
                                                    fake_net_assoc_data)
             self._assert_op(odl_const.ODL_UPDATE,
                             odl_const.ODL_BGPVPN,
                             fake_net_upd_bgpvpn_data)
-            self.thread.run_sync_thread(exit_after_run=True)
+            self.run_journal_processing()
             self._assert_op(odl_const.ODL_UPDATE,
                             odl_const.ODL_BGPVPN,
                             fake_net_upd_bgpvpn_data, False)
@@ -156,9 +146,12 @@ class OpendaylightBgpvpnDriverTestCase(base_v2.OpenDaylightConfigBase):
         fake_bgpvpn_data = self._get_fake_bgpvpn(net=False)
         with mock.patch.object(self.driver, 'get_bgpvpn',
                                return_value=fake_bgpvpn_data):
-            self.driver.delete_net_assoc_postcommit(self.context,
-                                                    fake_net_assoc_data)
-            self.thread.run_sync_thread(exit_after_run=True)
+            self.driver.delete_net_assoc_precommit(self.db_context,
+                                                   fake_net_assoc_data)
+            self._assert_op(odl_const.ODL_UPDATE,
+                            odl_const.ODL_BGPVPN,
+                            fake_bgpvpn_data)
+            self.run_journal_processing()
             self._assert_op(odl_const.ODL_UPDATE,
                             odl_const.ODL_BGPVPN,
                             fake_bgpvpn_data, False)
