@@ -17,15 +17,19 @@ test_l3_odl
 Tests for the L3 service plugin for networking-odl.
 """
 import copy
+
 import mock
 
 from neutron.extensions import l3
 from neutron.extensions import l3_ext_gw_mode
 from neutron.tests.unit.api.v2 import test_base
 from neutron.tests.unit.extensions import base as test_extensions_base
-from neutron_lib import constants
+from neutron_lib.plugins import constants
 from webob import exc
 
+from networking_odl.tests import base as odl_base
+
+
 _get_path = test_base._get_path
 
 
@@ -34,13 +38,14 @@ class Testodll3(test_extensions_base.ExtensionTestCase):
     fmt = 'json'
 
     def setUp(self):
+        self.useFixture(odl_base.OpenDaylightRestClientFixture())
         super(Testodll3, self).setUp()
         # support ext-gw-mode
         for key in l3.RESOURCE_ATTRIBUTE_MAP.keys():
             l3.RESOURCE_ATTRIBUTE_MAP[key].update(
                 l3_ext_gw_mode.EXTENDED_ATTRIBUTES_2_0.get(key, {}))
         self._setUpExtension(
-            'neutron.services.l3_router.l3_router_plugin.L3RouterPlugin',
+            'networking_odl.l3.l3_odl.OpenDaylightL3RouterPlugin',
             constants.L3, l3.RESOURCE_ATTRIBUTE_MAP,
             l3.L3, '', allow_pagination=True, allow_sorting=True,
             supported_extension_aliases=['router', 'ext-gw-mode'],
@@ -218,6 +223,7 @@ class Testodll3(test_extensions_base.ExtensionTestCase):
                              })
 
         instance = self.plugin.return_value
+        instance.get_floatingip = mock.Mock(return_value=floating_ip)
         instance.update_floatingip.return_value = return_value
         port_id, port = self._get_port_test()
         instance.get_port = mock.Mock(return_value=port)
@@ -245,8 +251,7 @@ class Testodll3(test_extensions_base.ExtensionTestCase):
         floating_ip_id, floating_ip = self._get_floating_ip_test()
 
         instance = self.plugin.return_value
-        port_id, port = self._get_port_test()
-        instance.get_port = mock.Mock(return_value=port)
+        instance.get_floatingip = mock.Mock(return_value=floating_ip)
         res = self.api.delete(_get_path('floatingips', id=floating_ip_id))
         instance.delete_floatingip.assert_called_once_with(mock.ANY,
                                                            floating_ip_id)
