@@ -16,38 +16,38 @@
 import mock
 import requests
 
-from neutron import context
-from neutron.db import api as neutron_db_api
 from neutron.extensions import external_net as external_net
 from neutron.plugins.ml2 import plugin
 from neutron.tests import base
 from neutron.tests.unit.db import test_db_base_plugin_v2
 from neutron.tests.unit import testlib_api
-from neutron_lib import constants
+from neutron_lib.plugins import constants
 from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_serialization import jsonutils
+from oslo_utils import uuidutils
 
 from networking_odl.common import client
 from networking_odl.common import constants as odl_const
 from networking_odl.common import filters
 from networking_odl.db import db
 from networking_odl.journal import journal
-from networking_odl.journal import maintenance
 from networking_odl.ml2 import mech_driver_v2
 from networking_odl.tests import base as odl_base
 from networking_odl.tests.unit import test_base_db
 
-EMPTY_DEP = []
-FLOATINGIP_ID = 'floatingip_uuid'
-NETWORK_ID = 'network_uuid'
-ROUTER_ID = 'router_uuid'
-SUBNET_ID = 'subnet_uuid'
-PORT_ID = 'port_uuid'
+EMPTY_DEP = {'gw_port_id': None}
+FLOATINGIP_ID = uuidutils.generate_uuid()
+NETWORK_ID = uuidutils.generate_uuid()
+ROUTER_ID = uuidutils.generate_uuid()
+SUBNET_ID = uuidutils.generate_uuid()
+PORT_ID = uuidutils.generate_uuid()
 
 
 class OpenDayLightMechanismConfigTests(testlib_api.SqlTestCase):
     def setUp(self):
+        self.useFixture(odl_base.OpenDaylightRestClientFixture())
+        self.useFixture(odl_base.OpenDaylightFeaturesFixture())
         super(OpenDayLightMechanismConfigTests, self).setUp()
         cfg.CONF.set_override('mechanism_drivers',
                               ['logger', 'opendaylight_v2'], 'ml2')
@@ -108,52 +108,47 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
         cfg.CONF.set_override("service_plugins", ['odl-router_v2'])
         core_plugin = cfg.CONF.core_plugin
         service_plugins = {'l3_plugin_name': 'odl-router_v2'}
-        mock.patch.object(journal.OpendaylightJournalThread,
-                          'start_odl_sync_thread').start()
-        self.mock_mt_thread = mock.patch.object(
-            maintenance.MaintenanceThread, 'start').start()
+        self.useFixture(odl_base.OpenDaylightJournalThreadFixture())
         mock.patch.object(mech_driver_v2.OpenDaylightMechanismDriver,
                           '_record_in_journal').start()
         mock.patch.object(mech_driver_v2.OpenDaylightMechanismDriver,
                           'sync_from_callback_precommit').start()
         mock.patch.object(mech_driver_v2.OpenDaylightMechanismDriver,
                           'sync_from_callback_postcommit').start()
+        self.useFixture(odl_base.OpenDaylightPeriodicTaskFixture())
+        self.useFixture(odl_base.OpenDaylightFeaturesFixture())
         super(OpenDaylightL3TestCase, self).setUp(
             plugin=core_plugin, service_plugins=service_plugins)
-        self.db_session = neutron_db_api.get_session()
         self.plugin = directory.get_plugin()
         self.plugin._network_is_external = mock.Mock(return_value=True)
         self.driver = directory.get_plugin(constants.L3)
-        self.thread = journal.OpendaylightJournalThread()
+        self.thread = journal.OpenDaylightJournalThread()
         self.driver.get_floatingip = mock.Mock(
             return_value={'router_id': ROUTER_ID,
                           'floating_network_id': NETWORK_ID})
 
     @staticmethod
     def _get_mock_router_operation_info(network, subnet):
-        router_context = context.get_admin_context()
         router = {odl_const.ODL_ROUTER:
                   {'name': 'router1',
                    'admin_state_up': True,
                    'tenant_id': network['network']['tenant_id'],
                    'external_gateway_info': {'network_id':
                                              network['network']['id']}}}
-        return router_context, router
+        return router
 
     @staticmethod
     def _get_mock_floatingip_operation_info(network, subnet):
-        floatingip_context = context.get_admin_context()
         floatingip = {odl_const.ODL_FLOATINGIP:
                       {'floating_network_id': network['network']['id'],
                        'tenant_id': network['network']['tenant_id']}}
-        return floatingip_context, floatingip
+        return floatingip
 
     @staticmethod
     def _get_mock_router_interface_operation_info(network, subnet):
-        router_intf_context = context.get_admin_context()
         router_intf_dict = {'subnet_id': subnet['subnet']['id'],
                             'id': network['network']['id']}
-        return router_intf_context, router_intf_dict
+        return router_intf_dict
 
     @classmethod
     def _get_mock_operation_info(cls, object_type, *args):
@@ -170,34 +165,30 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
 
     def _test_operation(self, status_code, expected_calls, *args, **kwargs):
         request_response = self._get_mock_request_response(status_code)
-        with mock.patch('requests.request',
+        with mock.patch('requests.sessions.Session.request',
                         return_value=request_response) as mock_method:
-            with mock.patch.object(self.thread.event, 'wait',
-                                   return_value=False):
-                self.thread.run_sync_thread(exit_after_run=True)
+            self.thread.sync_pending_entries()
 
         if expected_calls:
             mock_method.assert_called_with(
                 headers={'Content-Type': 'application/json'},
-                auth=(cfg.CONF.ml2_odl.username,
-                      cfg.CONF.ml2_odl.password),
                 timeout=cfg.CONF.ml2_odl.timeout, *args, **kwargs)
         self.assertEqual(expected_calls, mock_method.call_count)
 
     def _call_operation_object(self, operation, object_type, object_id,
                                network, subnet):
-        object_context, object_dict = self._get_mock_operation_info(
+        object_dict = self._get_mock_operation_info(
             object_type, network, subnet)
         method = getattr(self.driver, operation + '_' + object_type)
 
         if operation == odl_const.ODL_CREATE:
-            new_object_dict = method(object_context, object_dict)
+            new_object_dict = method(self.db_context, object_dict)
         elif operation == odl_const.ODL_UPDATE:
-            new_object_dict = method(object_context, object_id, object_dict)
+            new_object_dict = method(self.db_context, object_id, object_dict)
         else:
-            new_object_dict = method(object_context, object_id)
+            new_object_dict = method(self.db_context, object_id)
 
-        return object_context, new_object_dict
+        return new_object_dict
 
     def _test_operation_thread_processing(self, object_type, operation,
                                           network, subnet, object_id,
@@ -213,7 +204,7 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
         status_code = status_codes[operation]
 
         # Create database entry.
-        object_context, new_object_dict = self._call_operation_object(
+        new_object_dict = self._call_operation_object(
             operation, object_type, object_id, network, subnet)
 
         # Setup expected results.
@@ -282,13 +273,13 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
                   external_net.EXTERNAL: True}
         with self.network(**kwargs) as network:
             with self.subnet(network=network):
-                object_context, object_dict = self._get_mock_operation_info(
+                object_dict = self._get_mock_operation_info(
                     object_type, network, None)
 
                 # Add and test 'create' database entry.
                 method = getattr(self.driver,
                                  odl_const.ODL_CREATE + '_' + object_type)
-                new_object_dict = method(object_context, object_dict)
+                new_object_dict = method(self.db_context, object_dict)
                 object_id = new_object_dict['id']
                 self._test_db_results(object_id, odl_const.ODL_CREATE,
                                       object_type)
@@ -296,20 +287,20 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
                 # Add and test 'update' database entry.
                 method = getattr(self.driver,
                                  odl_const.ODL_UPDATE + '_' + object_type)
-                method(object_context, object_id, object_dict)
+                method(self.db_context, object_id, object_dict)
                 self._test_db_results(object_id, odl_const.ODL_UPDATE,
                                       object_type)
 
                 # Add and test 'delete' database entry.
                 method = getattr(self.driver,
                                  odl_const.ODL_DELETE + '_' + object_type)
-                method(object_context, object_id)
+                method(self.db_context, object_id)
                 self._test_db_results(object_id, odl_const.ODL_DELETE,
                                       object_type)
 
     def _test_dependency_processing(
-            self, test_operation, test_object, test_id, test_context,
-            dep_operation, dep_object, dep_id, dep_context):
+            self, test_operation, test_object, test_id, test_data,
+            dep_operation, dep_object, dep_id, dep_data):
 
         # Mock sendjson to verify that it never gets called.
         mock_sendjson = mock.patch.object(client.OpenDaylightRestClient,
@@ -317,19 +308,16 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
 
         # Create dependency db row and mark as 'processing' so it won't
         # be processed by the journal thread.
-        db.create_pending_row(self.db_session, dep_object,
-                              dep_id, dep_operation, dep_context)
+        ctxt = self.db_context
+        journal.record(ctxt, dep_object, dep_id, dep_operation, dep_data)
         row = db.get_all_db_rows_by_state(self.db_session, odl_const.PENDING)
         db.update_db_row_state(self.db_session, row[0], odl_const.PROCESSING)
 
         # Create test row with dependent ID.
-        db.create_pending_row(self.db_session, test_object,
-                              test_id, test_operation, test_context)
+        journal.record(ctxt, test_object, test_id, test_operation, test_data)
 
         # Call journal thread.
-        with mock.patch.object(self.thread.event, 'wait',
-                               return_value=False):
-            self.thread.run_sync_thread(exit_after_run=True)
+        self.thread.sync_pending_entries()
 
         # Verify that dependency row is still set at 'processing'.
         rows = db.get_all_db_rows_by_state(self.db_session,
@@ -368,7 +356,9 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
         self._test_dependency_processing(
             odl_const.ODL_CREATE, odl_const.ODL_ROUTER, ROUTER_ID,
             router_context,
-            odl_const.ODL_CREATE, odl_const.ODL_PORT, PORT_ID, None)
+            odl_const.ODL_CREATE, odl_const.ODL_PORT, PORT_ID,
+            {'fixed_ips': [], 'network_id': None, odl_const.ODL_SGS: None,
+             'tenant_id': 'tenant'})
 
     def test_delete_router_validate_ext_delete_floatingip_dep(self):
         floatingip_context = [ROUTER_ID]
@@ -380,26 +370,24 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
     def test_delete_router_validate_self_create_dep(self):
         self._test_dependency_processing(
             odl_const.ODL_DELETE, odl_const.ODL_ROUTER, ROUTER_ID, EMPTY_DEP,
-            odl_const.ODL_CREATE, odl_const.ODL_ROUTER, ROUTER_ID, None)
+            odl_const.ODL_CREATE, odl_const.ODL_ROUTER, ROUTER_ID, EMPTY_DEP)
 
     def test_delete_router_validate_self_update_dep(self):
         self._test_dependency_processing(
             odl_const.ODL_DELETE, odl_const.ODL_ROUTER, ROUTER_ID, EMPTY_DEP,
-            odl_const.ODL_UPDATE, odl_const.ODL_ROUTER, ROUTER_ID, None)
+            odl_const.ODL_UPDATE, odl_const.ODL_ROUTER, ROUTER_ID, EMPTY_DEP)
 
     def test_update_router_validate_self_create_dep(self):
-        router_context = {'gw_port_id': None}
         self._test_dependency_processing(
-            odl_const.ODL_UPDATE, odl_const.ODL_ROUTER, ROUTER_ID,
-            router_context,
-            odl_const.ODL_CREATE, odl_const.ODL_ROUTER, ROUTER_ID, None)
+            odl_const.ODL_UPDATE, odl_const.ODL_ROUTER, ROUTER_ID, EMPTY_DEP,
+            odl_const.ODL_CREATE, odl_const.ODL_ROUTER, ROUTER_ID, EMPTY_DEP)
 
     def test_create_floatingip_validate_ext_create_network_dep(self):
         floatingip_context = {'floating_network_id': NETWORK_ID}
         self._test_dependency_processing(
             odl_const.ODL_CREATE, odl_const.ODL_FLOATINGIP, FLOATINGIP_ID,
             floatingip_context,
-            odl_const.ODL_CREATE, odl_const.ODL_NETWORK, NETWORK_ID, None)
+            odl_const.ODL_CREATE, odl_const.ODL_NETWORK, NETWORK_ID, {})
 
     def test_update_floatingip_validate_self_create_dep(self):
         floatingip_context = {'floating_network_id': NETWORK_ID}
@@ -414,11 +402,11 @@ class OpenDaylightL3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
             odl_const.ODL_DELETE, odl_const.ODL_FLOATINGIP, FLOATINGIP_ID,
             EMPTY_DEP,
             odl_const.ODL_CREATE, odl_const.ODL_FLOATINGIP, FLOATINGIP_ID,
-            None)
+            {})
 
     def test_delete_floatingip_validate_self_update_dep(self):
         self._test_dependency_processing(
             odl_const.ODL_DELETE, odl_const.ODL_FLOATINGIP, FLOATINGIP_ID,
             EMPTY_DEP,
             odl_const.ODL_UPDATE, odl_const.ODL_FLOATINGIP, FLOATINGIP_ID,
-            None)
+            {})
