@@ -15,11 +15,11 @@
 
 import mock
 
-from neutron.plugins.common import constants
-from neutron.plugins.ml2 import driver_api as api
+from neutron.plugins.ml2 import driver_api
 from neutron.plugins.ml2 import driver_context as ctx
 from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants as n_constants
+from neutron_lib.plugins.ml2 import api
 
 from networking_odl.ml2 import legacy_port_binding
 from networking_odl.tests import base
@@ -29,22 +29,22 @@ class TestLegacyPortBindingManager(base.DietTestCase):
     # valid  and invalid segments
     valid_segment = {
         api.ID: 'API_ID',
-        api.NETWORK_TYPE: constants.TYPE_LOCAL,
+        api.NETWORK_TYPE: n_constants.TYPE_LOCAL,
         api.SEGMENTATION_ID: 'API_SEGMENTATION_ID',
         api.PHYSICAL_NETWORK: 'API_PHYSICAL_NETWORK'}
 
     invalid_segment = {
         api.ID: 'API_ID',
-        api.NETWORK_TYPE: constants.TYPE_NONE,
+        api.NETWORK_TYPE: n_constants.TYPE_NONE,
         api.SEGMENTATION_ID: 'API_SEGMENTATION_ID',
         api.PHYSICAL_NETWORK: 'API_PHYSICAL_NETWORK'}
 
     def test_check_segment(self):
         """Validate the _check_segment method."""
 
-        all_network_types = [constants.TYPE_FLAT, constants.TYPE_GRE,
-                             constants.TYPE_LOCAL, constants.TYPE_VXLAN,
-                             constants.TYPE_VLAN, constants.TYPE_NONE]
+        all_network_types = [n_constants.TYPE_FLAT, n_constants.TYPE_GRE,
+                             n_constants.TYPE_LOCAL, n_constants.TYPE_VXLAN,
+                             n_constants.TYPE_VLAN, n_constants.TYPE_NONE]
 
         mgr = legacy_port_binding.LegacyPortBindingManager()
 
@@ -54,13 +54,13 @@ class TestLegacyPortBindingManager(base.DietTestCase):
             if mgr._check_segment({api.NETWORK_TYPE: network_type})}
 
         self.assertEqual({
-            constants.TYPE_FLAT, constants.TYPE_LOCAL,
-            constants.TYPE_GRE, constants.TYPE_VXLAN,
-            constants.TYPE_VLAN}, valid_types)
+            n_constants.TYPE_FLAT, n_constants.TYPE_LOCAL,
+            n_constants.TYPE_GRE, n_constants.TYPE_VXLAN,
+            n_constants.TYPE_VLAN}, valid_types)
 
     def test_bind_port(self):
 
-        network = mock.MagicMock(spec=api.NetworkContext)
+        network = mock.MagicMock(spec=driver_api.NetworkContext)
 
         port_context = mock.MagicMock(
             spec=ctx.PortContext, current={'id': 'CURRENT_CONTEXT_ID'},
@@ -77,7 +77,7 @@ class TestLegacyPortBindingManager(base.DietTestCase):
             mgr.vif_details, status=n_constants.PORT_STATUS_ACTIVE)
 
     def test_bind_port_unsupported_vnic_type(self):
-        network = mock.MagicMock(spec=api.NetworkContext)
+        network = mock.MagicMock(spec=driver_api.NetworkContext)
         port_context = mock.MagicMock(
             spec=ctx.PortContext,
             current={'id': 'CURRENT_CONTEXT_ID',
