@@ -15,8 +15,8 @@
 
 
 import copy
+
 import mock
-import socket
 import testscenarios
 
 from oslo_config import cfg
@@ -26,10 +26,10 @@ import webob.exc
 
 from neutron.db import segments_db
 from neutron.extensions import multiprovidernet as mpnet
-from neutron.plugins.common import constants
 from neutron.plugins.ml2 import config
 from neutron.plugins.ml2 import driver_api as api
 from neutron.plugins.ml2 import driver_context as driver_context
+from neutron.plugins.ml2 import models
 from neutron.plugins.ml2 import plugin
 from neutron.tests import base
 from neutron.tests.unit.plugins.ml2 import test_plugin
@@ -37,12 +37,12 @@ from neutron.tests.unit import testlib_api
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net as providernet
 from neutron_lib import constants as n_constants
+from neutron_lib.plugins.ml2 import api as ml2_api
 
 from networking_odl.common import client
 from networking_odl.common import constants as odl_const
 from networking_odl.ml2 import legacy_port_binding
 from networking_odl.ml2 import mech_driver
-from networking_odl.ml2 import network_topology
 from networking_odl.tests import base as odl_base
 
 
@@ -123,26 +123,14 @@ class OpenDaylightTestCase(test_plugin.Ml2PluginV2TestCase):
 
     def setUp(self):
         self.useFixture(odl_base.OpenDaylightRestClientFixture())
-        super(OpenDaylightTestCase, self).setUp()
-        self.port_create_status = 'DOWN'
-        self.mech = mech_driver.OpenDaylightMechanismDriver()
+        self.useFixture(odl_base.OpenDaylightFeaturesFixture())
         mock.patch.object(
             client.OpenDaylightRestClient,
             'sendjson',
             new=self.check_sendjson).start()
-
-        # Prevent test from accidentally connecting to any web service
-        mock.patch.object(
-            network_topology, 'NetworkTopologyClient',
-            return_value=mock.Mock(
-                specs=network_topology.NetworkTopologyClient,
-                get=mock.Mock(side_effect=requests.HTTPError))).start()
-
-        # Prevent hosts resolution from changing the behaviour of tests
-        mock.patch.object(
-            network_topology.utils,
-            'get_addresses_by_name',
-            side_effect=socket.gaierror).start()
+        super(OpenDaylightTestCase, self).setUp()
+        self.port_create_status = 'DOWN'
+        self.mech = mech_driver.OpenDaylightMechanismDriver()
 
     def check_sendjson(self, method, urlpath, obj):
         self.assertFalse(urlpath.startswith("http://"))
@@ -150,6 +138,7 @@ class OpenDaylightTestCase(test_plugin.Ml2PluginV2TestCase):
 
 class OpenDayLightMechanismConfigTests(testlib_api.SqlTestCase):
     def setUp(self):
+        self.useFixture(odl_base.OpenDaylightFeaturesFixture())
         super(OpenDayLightMechanismConfigTests, self).setUp()
         config.cfg.CONF.set_override('mechanism_drivers',
                                      ['logger', 'opendaylight'],
@@ -244,6 +233,7 @@ class OpenDaylightSyncTestCase(OpenDaylightTestCase):
     def test_simple_sync_all_with_HTTPError_not_found(self):
         self.given_back_end.out_of_sync = True
         ml2_plugin = plugin.Ml2Plugin()
+        port_mock = mock.MagicMock(port_binding=models.PortBinding())
 
         response = mock.Mock(status_code=requests.codes.not_found)
         fake_exception = requests.exceptions.HTTPError('Test',
@@ -263,6 +253,8 @@ class OpenDaylightSyncTestCase(OpenDaylightTestCase):
                               return_value=FAKE_NETWORK.copy()), \
             mock.patch.object(plugin.Ml2Plugin, 'get_subnets',
                               return_value=[FAKE_SUBNET.copy()]), \
+            mock.patch.object(plugin.Ml2Plugin, '_get_port',
+                              return_value=port_mock), \
             mock.patch.object(plugin.Ml2Plugin, 'get_ports',
                               return_value=[FAKE_PORT.copy()]), \
             mock.patch.object(plugin.Ml2Plugin, 'get_security_groups',
@@ -311,8 +303,9 @@ class OpenDaylightSyncTestCase(OpenDaylightTestCase):
 class OpenDaylightMechanismDriverTestCase(base.BaseTestCase):
 
     def setUp(self):
-        super(OpenDaylightMechanismDriverTestCase, self).setUp()
         self.useFixture(odl_base.OpenDaylightRestClientFixture())
+        self.useFixture(odl_base.OpenDaylightFeaturesFixture())
+        super(OpenDaylightMechanismDriverTestCase, self).setUp()
         config.cfg.CONF.set_override('mechanism_drivers',
                                      ['logger', 'opendaylight'], 'ml2')
         self.mech = mech_driver.OpenDaylightMechanismDriver()
@@ -364,7 +357,7 @@ class OpenDaylightMechanismDriverTestCase(base.BaseTestCase):
                                exc_class=None, *args, **kwargs):
         self.mech.odl_drv.out_of_sync = False
         request_response = self._get_mock_request_response(status_code)
-        with mock.patch('requests.request',
+        with mock.patch('requests.sessions.Session.request',
                         return_value=request_response) as mock_method:
             if exc_class is not None:
                 self.assertRaises(exc_class, method, context)
@@ -372,8 +365,6 @@ class OpenDaylightMechanismDriverTestCase(base.BaseTestCase):
                 method(context)
         mock_method.assert_called_once_with(
             headers={'Content-Type': 'application/json'},
-            auth=(config.cfg.CONF.ml2_odl.username,
-                  config.cfg.CONF.ml2_odl.password),
             timeout=config.cfg.CONF.ml2_odl.timeout, *args, **kwargs)
 
     def _test_create_resource_postcommit(self, object_type, status_code,
@@ -524,10 +515,11 @@ class OpenDaylightMechanismDriverTestCase(base.BaseTestCase):
         port = self._get_mock_operation_context(odl_const.ODL_PORT).current
         tenant_id = network['tenant_id']
         port['tenant_id'] = ''
+        binding = models.PortBinding()
 
         with mock.patch.object(segments_db, 'get_network_segments'):
             context = driver_context.PortContext(
-                plugin, plugin_context, port, network, {}, 0, None)
+                plugin, plugin_context, port, network, binding, 0, None)
             self.mech.odl_drv.FILTER_MAP[
                 odl_const.ODL_PORTS].filter_create_attributes(port, context)
             self.assertEqual(tenant_id, port['tenant_id'])
@@ -544,10 +536,11 @@ class OpenDaylightMechanismDriverTestCase(base.BaseTestCase):
                               'ip_address': '10.0.0.10'}]
         port['mac_address'] = port['mac_address'].upper()
         orig_port = copy.deepcopy(port)
+        binding = models.PortBinding()
 
         with mock.patch.object(segments_db, 'get_network_segments'):
             context = driver_context.PortContext(
-                plugin, plugin_context, port, network, {}, 0, None)
+                plugin, plugin_context, port, network, binding, 0, None)
             self.mech.odl_drv.FILTER_MAP[
                 odl_const.ODL_PORTS].filter_update_attributes(port, context)
             for key, value in port.items():
@@ -558,22 +551,23 @@ class OpenDaylightMechanismDriverTestCase(base.BaseTestCase):
 class TestOpenDaylightMechanismDriver(base.DietTestCase):
     def setUp(self):
         self.useFixture(odl_base.OpenDaylightRestClientFixture())
+        self.useFixture(odl_base.OpenDaylightFeaturesFixture())
         super(TestOpenDaylightMechanismDriver, self).setUp()
         config.cfg.CONF.set_override('mechanism_drivers',
                                      ['logger', 'opendaylight'], 'ml2')
 
     # given valid  and invalid segments
     valid_segment = {
-        api.ID: 'API_ID',
-        api.NETWORK_TYPE: constants.TYPE_LOCAL,
-        api.SEGMENTATION_ID: 'API_SEGMENTATION_ID',
-        api.PHYSICAL_NETWORK: 'API_PHYSICAL_NETWORK'}
+        ml2_api.ID: 'API_ID',
+        ml2_api.NETWORK_TYPE: n_constants.TYPE_LOCAL,
+        ml2_api.SEGMENTATION_ID: 'API_SEGMENTATION_ID',
+        ml2_api.PHYSICAL_NETWORK: 'API_PHYSICAL_NETWORK'}
 
     invalid_segment = {
-        api.ID: 'API_ID',
-        api.NETWORK_TYPE: constants.TYPE_NONE,
-        api.SEGMENTATION_ID: 'API_SEGMENTATION_ID',
-        api.PHYSICAL_NETWORK: 'API_PHYSICAL_NETWORK'}
+        ml2_api.ID: 'API_ID',
+        ml2_api.NETWORK_TYPE: n_constants.TYPE_NONE,
+        ml2_api.SEGMENTATION_ID: 'API_SEGMENTATION_ID',
+        ml2_api.PHYSICAL_NETWORK: 'API_PHYSICAL_NETWORK'}
 
     def test_bind_port_front_end(self):
         given_front_end = mech_driver.OpenDaylightMechanismDriver()
@@ -589,7 +583,7 @@ class TestOpenDaylightMechanismDriver(base.DietTestCase):
         # then context binding is setup with returned vif_type and valid
         # segment API ID
         given_port_context.set_binding.assert_called_once_with(
-            self.valid_segment[api.ID], portbindings.VIF_TYPE_OVS,
+            self.valid_segment[ml2_api.ID], portbindings.VIF_TYPE_OVS,
             given_back_end.port_binding_controller.vif_details,
             status=n_constants.PORT_STATUS_ACTIVE)
 
@@ -629,11 +623,11 @@ class TestOpenDaylightDriverVlanTransparencyNetwork(
         self.assertEqual(True,
                          self.mech.check_vlan_transparency(context))
 
-        for network_type in [constants.TYPE_VXLAN]:
+        for network_type in [n_constants.TYPE_VXLAN]:
             self._test_network_type(True, network_type)
-        for network_type in [constants.TYPE_FLAT, constants.TYPE_GENEVE,
-                             constants.TYPE_GRE, constants.TYPE_LOCAL,
-                             constants.TYPE_VLAN]:
+        for network_type in [n_constants.TYPE_FLAT, n_constants.TYPE_GENEVE,
+                             n_constants.TYPE_GRE, n_constants.TYPE_LOCAL,
+                             n_constants.TYPE_VLAN]:
             self._test_network_type(False, network_type)
 
 
@@ -642,19 +636,19 @@ class TestOpenDaylightDriverVlanTransparency(
     scenarios = [
         ('vxlan_vxlan',
          {'expected': True,
-          'network_types': [constants.TYPE_VXLAN, constants.TYPE_VXLAN]}),
+          'network_types': [n_constants.TYPE_VXLAN, n_constants.TYPE_VXLAN]}),
         ('gre_vxlan',
          {'expected': False,
-          'network_types': [constants.TYPE_GRE, constants.TYPE_VXLAN]}),
+          'network_types': [n_constants.TYPE_GRE, n_constants.TYPE_VXLAN]}),
         ('vxlan_vlan',
          {'expected': False,
-          'network_types': [constants.TYPE_VXLAN, constants.TYPE_VLAN]}),
+          'network_types': [n_constants.TYPE_VXLAN, n_constants.TYPE_VLAN]}),
         ('vxlan_flat',
          {'expected': False,
-          'network_types': [constants.TYPE_VXLAN, constants.TYPE_FLAT]}),
+          'network_types': [n_constants.TYPE_VXLAN, n_constants.TYPE_FLAT]}),
         ('vlan_vlan',
          {'expected': False,
-          'network_types': [constants.TYPE_VLAN, constants.TYPE_VLAN]}),
+          'network_types': [n_constants.TYPE_VLAN, n_constants.TYPE_VLAN]}),
     ]
 
     def test_network_segments(self):
