@@ -14,22 +14,24 @@
 #    under the License.
 import copy
 import datetime
-import mock
 import operator
+
+import mock
 import requests
 import testscenarios
 
-from neutron.db import api as neutron_db_api
 from neutron.db.models import securitygroup
+from neutron.db import segments_db
 from neutron.extensions import multiprovidernet as mpnet
-from neutron.plugins.common import constants as p_const
 from neutron.plugins.ml2 import plugin
 from neutron.tests.unit.plugins.ml2 import test_plugin
 from neutron.tests.unit import testlib_api
 from neutron_lib.api.definitions import provider_net as providernet
+from neutron_lib import constants as n_constants
 from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_serialization import jsonutils
+from oslo_utils import uuidutils
 
 from networking_odl.common import callback
 from networking_odl.common import constants as odl_const
@@ -38,8 +40,8 @@ from networking_odl.common import utils
 from networking_odl.db import db
 from networking_odl.journal import cleanup
 from networking_odl.journal import journal
-from networking_odl.journal import maintenance
 from networking_odl.ml2 import mech_driver_v2
+from networking_odl.tests import base
 from networking_odl.tests.unit import base_v2
 
 
@@ -49,17 +51,15 @@ load_tests = testscenarios.load_tests_apply_scenarios
 cfg.CONF.import_group('ml2_odl', 'networking_odl.common.config')
 
 SECURITY_GROUP = '2f9244b4-9bee-4e81-bc4a-3f3c2045b3d7'
-SG_FAKE_ID = 'sg_fake_uuid'
-SG_RULE_FAKE_ID = 'sg_rule_fake_uuid'
+SG_FAKE_ID = uuidutils.generate_uuid()
+SG_RULE_FAKE_ID = uuidutils.generate_uuid()
 
 
 class OpenDayLightMechanismConfigTests(testlib_api.SqlTestCase):
     def setUp(self):
         super(OpenDayLightMechanismConfigTests, self).setUp()
-        self.mock_sync_thread = mock.patch.object(
-            journal.OpendaylightJournalThread, 'start_odl_sync_thread').start()
-        self.mock_mt_thread = mock.patch.object(
-            maintenance.MaintenanceThread, 'start').start()
+        self.useFixture(base.OpenDaylightFeaturesFixture())
+        self.useFixture(base.OpenDaylightJournalThreadFixture())
         cfg.CONF.set_override('mechanism_drivers',
                               ['logger', 'opendaylight_v2'], 'ml2')
         cfg.CONF.set_override('port_binding_controller',
@@ -150,14 +150,13 @@ class AttributeDict(dict):
 
 class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
     def setUp(self):
+        self.useFixture(base.OpenDaylightFeaturesFixture())
+        self.useFixture(base.OpenDaylightJournalThreadFixture())
         super(OpenDaylightMechanismDriverTestCase, self).setUp()
-        self.db_session = neutron_db_api.get_session()
         self.mech = mech_driver_v2.OpenDaylightMechanismDriver()
         self.mech.initialize()
-        self.thread = journal.OpendaylightJournalThread()
 
-    @staticmethod
-    def _get_mock_network_operation_context():
+    def _get_mock_network_operation_context(self):
         current = {'status': 'ACTIVE',
                    'subnets': [],
                    'name': 'net1',
@@ -170,11 +169,10 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
                    'id': 'd897e21a-dfd6-4331-a5dd-7524fa421c3e',
                    'provider:segmentation_id': None}
         context = mock.Mock(current=current)
-        context._plugin_context.session = neutron_db_api.get_session()
+        context._plugin_context = self.db_context
         return context
 
-    @staticmethod
-    def _get_mock_subnet_operation_context():
+    def _get_mock_subnet_operation_context(self):
         current = {'ipv6_ra_mode': None,
                    'allocation_pools': [{'start': '10.0.0.2',
                                          'end': '10.0.1.254'}],
@@ -191,11 +189,10 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
                    'ip_version': 4,
                    'shared': False}
         context = mock.Mock(current=current)
-        context._plugin_context.session = neutron_db_api.get_session()
+        context._plugin_context = self.db_context
         return context
 
-    @staticmethod
-    def _get_mock_port_operation_context():
+    def _get_mock_port_operation_context(self):
         current = {'status': 'DOWN',
                    'binding:host_id': '',
                    'allowed_address_pairs': [],
@@ -214,13 +211,13 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
                    'binding:vnic_type': 'normal',
                    'binding:vif_type': 'unbound',
                    'mac_address': '12:34:56:78:21:b6'}
-        _network = OpenDaylightMechanismDriverTestCase.\
-            _get_mock_network_operation_context().current
+        _network = self._get_mock_network_operation_context().current
         _plugin = directory.get_plugin()
-        _plugin.get_security_group = mock.Mock(return_value=SECURITY_GROUP)
+        _plugin.writer_get_security_group = mock.Mock(
+            return_value=SECURITY_GROUP)
         _plugin.get_port = mock.Mock(return_value=current)
         _plugin.get_network = mock.Mock(return_value=_network)
-        _plugin_context_mock = {'session': neutron_db_api.get_session()}
+        _plugin_context_mock = {'session': self.db_session}
         _network_context_mock = {'_network': _network}
         context = {'current': AttributeDict(current),
                    '_plugin': _plugin,
@@ -228,25 +225,25 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
                    '_network_context': AttributeDict(_network_context_mock)}
         return AttributeDict(context)
 
-    @staticmethod
-    def _get_mock_security_group_operation_context():
+    def _get_mock_security_group_operation_context(self):
         context = {odl_const.ODL_SG: {'name': 'test_sg',
                                       'project_id': 'test-tenant',
                                       'tenant_id': 'test-tenant',
                                       'description': 'test-description',
-                                      'rules': [],
+                                      'security_group_rules': [],
                                       'id': SG_FAKE_ID}}
         return context
 
-    @staticmethod
-    def _get_mock_security_group_rule_operation_context():
+    def _get_mock_security_group_rule_operation_context(self):
         context = {odl_const.ODL_SG_RULE: {'security_group_id': SG_FAKE_ID,
                                            'id': SG_RULE_FAKE_ID}}
+        _plugin = directory.get_plugin()
+        _plugin._get_security_group_rule = mock.Mock(
+            return_value=AttributeDict(context[odl_const.ODL_SG_RULE]))
         return context
 
-    @classmethod
-    def _get_mock_operation_context(cls, object_type):
-        getter = getattr(cls, '_get_mock_%s_operation_context' % object_type)
+    def _get_mock_operation_context(self, object_type):
+        getter = getattr(self, '_get_mock_%s_operation_context' % object_type)
         return getter()
 
     _status_code_msgs = {
@@ -270,31 +267,28 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
                 cls._status_code_msgs[status_code])))
         return response
 
-    def _test_operation(self, method, status_code, expected_calls,
+    def _test_operation(self, status_code, expected_calls,
                         *args, **kwargs):
         request_response = self._get_mock_request_response(status_code)
-        with mock.patch('requests.request',
+        with mock.patch('requests.sessions.Session.request',
                         return_value=request_response) as mock_method:
-            method(exit_after_run=True)
+            self.run_journal_processing()
 
         if expected_calls:
             mock_method.assert_called_with(
                 headers={'Content-Type': 'application/json'},
-                auth=(cfg.CONF.ml2_odl.username, cfg.CONF.ml2_odl.password),
                 timeout=cfg.CONF.ml2_odl.timeout, *args, **kwargs)
         self.assertEqual(expected_calls, mock_method.call_count)
 
     def _call_operation_object(self, operation, object_type):
         context = self._get_mock_operation_context(object_type)
-
         if object_type in [odl_const.ODL_SG, odl_const.ODL_SG_RULE]:
-            plugin_context_mock = mock.Mock()
-            plugin_context_mock.session = neutron_db_api.get_session()
             res_type = [rt for rt in callback._RESOURCE_MAPPING.values()
                         if rt.singular == object_type][0]
             res_id = context[object_type]['id']
             context_ = (copy.deepcopy(context)
                         if operation != odl_const.ODL_DELETE else None)
+            plugin_context = self.db_context
             if (object_type == odl_const.ODL_SG and
                     operation in [odl_const.ODL_CREATE, odl_const.ODL_DELETE]):
                 # TODO(yamahata): remove this work around once
@@ -305,21 +299,31 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
                         id=res_id, name=context_[object_type]['name'],
                         tenant_id=context_[object_type]['tenant_id'],
                         description=context_[object_type]['description'])
-                    plugin_context_mock.session.add(sg)
-                    context_[odl_const.ODL_SG].pop('id', None)
+                    plugin_context.session.add(sg)
+                    sg_dict = dict(sg)
+                    sg_dict['security_group_rules'] = []
+                    with self.db_session.begin(subtransactions=True):
+                        self.mech.sync_from_callback_precommit(
+                            plugin_context, operation, res_type, res_id,
+                            context_, security_group=sg_dict)
                 if operation == odl_const.ODL_DELETE:
-                    sg = mock.Mock()
-                    sg.rules = []
-                self.mech.sync_from_callback_precommit(
-                    plugin_context_mock, operation, res_type, res_id, context_,
-                    security_group=sg)
+                    with self.db_session.begin(subtransactions=True):
+                        self.mech.sync_from_callback_precommit(
+                            plugin_context, operation, res_type, res_id,
+                            context_,
+                            security_group={'security_group_rules':
+                                            {'id': SG_RULE_FAKE_ID}},
+                            security_group_rule_ids=[SG_RULE_FAKE_ID])
             else:
-                self.mech.sync_from_callback_precommit(
-                    plugin_context_mock, operation, res_type, res_id, context_)
+                with self.db_session.begin(subtransactions=True):
+                    self.mech.sync_from_callback_precommit(
+                        plugin_context, operation, res_type, res_id,
+                        context_)
         else:
             method = getattr(self.mech, '%s_%s_precommit' % (operation,
                                                              object_type))
-            method(context)
+            with self.db_session.begin(subtransactions=True):
+                method(context)
 
     def _test_operation_object(self, operation, object_type):
         self._call_operation_object(operation, object_type)
@@ -329,6 +333,7 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
         self.assertEqual(operation, row['operation'])
         self.assertEqual(object_type, row['object_type'])
         self.assertEqual(context.current['id'], row['object_uuid'])
+        self._db_cleanup()
 
     def _test_thread_processing(self, operation, object_type,
                                 expected_calls=1):
@@ -358,19 +363,17 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
         if (object_type == odl_const.ODL_SG and
                 operation == odl_const.ODL_CREATE):
             context = copy.deepcopy(context)
-            context[odl_const.ODL_SG].pop('rules')
         if operation in [odl_const.ODL_CREATE, odl_const.ODL_UPDATE]:
             kwargs = {
                 'url': url,
                 'data': DataMatcher(operation, object_type, context)}
         else:
             kwargs = {'url': url, 'data': None}
-        with mock.patch.object(self.thread.event, 'wait',
-                               return_value=False):
-            self._test_operation(self.thread.run_sync_thread, status_code,
-                                 expected_calls, http_request, **kwargs)
 
-    def _test_object_type(self, object_type):
+        self._test_operation(status_code, expected_calls, http_request,
+                             **kwargs)
+
+    def _test_object_type(self, object_type, delete_expected_calls=1):
         # Add and process create request.
         self._test_thread_processing(odl_const.ODL_CREATE, object_type)
         rows = db.get_all_db_rows_by_state(self.db_session,
@@ -384,10 +387,11 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
         self.assertEqual(2, len(rows))
 
         # Add and process update request. Adds to database.
-        self._test_thread_processing(odl_const.ODL_DELETE, object_type)
+        self._test_thread_processing(odl_const.ODL_DELETE, object_type,
+                                     delete_expected_calls)
         rows = db.get_all_db_rows_by_state(self.db_session,
                                            odl_const.COMPLETED)
-        self.assertEqual(3, len(rows))
+        self.assertEqual(2 + delete_expected_calls, len(rows))
 
     def _test_object_type_pending_network(self, object_type):
         # Create a network (creates db row in pending state).
@@ -457,7 +461,7 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
 
         # Test if the cleanup marks this in the desired state
         # based on the last_retried timestamp
-        cleanup.JournalCleanup().cleanup_processing_rows(self.db_session)
+        cleanup.JournalCleanup().cleanup_processing_rows(self.db_context)
 
         # Verify that the Db row is in the desired state
         rows = db.get_all_db_rows_by_state(self.db_session, expected_state)
@@ -476,12 +480,12 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
 
         method = getattr(self.mech, 'create_port_precommit')
         method(context)
+        self.db_session.flush()
 
         # Verify that the Db row has a tenant
         rows = db.get_all_db_rows_by_state(self.db_session, odl_const.PENDING)
         self.assertEqual(1, len(rows))
-        _network = OpenDaylightMechanismDriverTestCase.\
-            _get_mock_network_operation_context().current
+        _network = self._get_mock_network_operation_context().current
         self.assertEqual(_network['tenant_id'], rows[0]['data']['tenant_id'])
 
     def test_network(self):
@@ -564,16 +568,22 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
     def test_thread_call(self):
         """Verify that the sync thread method is called."""
 
-        # Create any object that would spin up the sync thread via the
-        # decorator call_thread_on_end() used by all the event handlers.
-        self._call_operation_object(odl_const.ODL_CREATE,
-                                    odl_const.ODL_NETWORK)
+        with mock.patch.object(
+                journal.OpenDaylightJournalThread,
+                'start') as mock_sync_thread:
+            self.mech = mech_driver_v2.OpenDaylightMechanismDriver()
+            self.mech.initialize()
 
-        # Verify that the thread call was made.
-        self.assertTrue(self.mock_sync_thread.called)
+            # Create any object that would spin up the sync thread via the
+            # decorator call_thread_on_end() used by all the event handlers.
+            self._call_operation_object(odl_const.ODL_CREATE,
+                                        odl_const.ODL_NETWORK)
+
+            # Verify that the thread call was made.
+            mock_sync_thread.assert_called()
 
     def test_sg(self):
-        self._test_object_type(odl_const.ODL_SG)
+        self._test_object_type(odl_const.ODL_SG, 2)
 
     def test_sg_rule(self):
         self._test_object_type(odl_const.ODL_SG_RULE)
@@ -582,29 +592,29 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
         with mock.patch.object(journal, 'record') as record:
             context = self._get_mock_operation_context(odl_const.ODL_SG)
             res_id = context[odl_const.ODL_SG]['id']
-            plugin_context_mock = mock.Mock()
-            plugin_context_mock.session = neutron_db_api.get_session()
             rule = mock.Mock()
             rule.id = SG_RULE_FAKE_ID
             rule.security_group_id = SG_FAKE_ID
             sg = mock.Mock()
             sg.id = SG_FAKE_ID
-            sg.rules = [rule]
-            kwargs = {'security_group': sg}
-            self.mech.sync_from_callback_precommit(
-                plugin_context_mock, odl_const.ODL_DELETE,
-                callback._RESOURCE_MAPPING[odl_const.ODL_SG],
-                res_id, context, **kwargs)
+            sg.security_group_rules = [rule]
+            kwargs = {'security_group': sg,
+                      'security_group_rule_ids': [SG_RULE_FAKE_ID]}
+            with self.db_session.begin(subtransactions=True):
+                self.mech.sync_from_callback_precommit(
+                    self.db_context, odl_const.ODL_DELETE,
+                    callback._RESOURCE_MAPPING[odl_const.ODL_SG],
+                    res_id, context, **kwargs)
             record.assert_has_calls(
-                [mock.call(mock.ANY, None, 'security_group', 'sg_fake_uuid',
+                [mock.call(mock.ANY, 'security_group_rule',
+                           SG_RULE_FAKE_ID, 'delete', [SG_FAKE_ID]),
+                 mock.call(mock.ANY, 'security_group', SG_FAKE_ID,
                            'delete',
                            {'description': 'test-description',
                             'project_id': 'test-tenant',
-                            'rules': [],
+                            'security_group_rules': [],
                             'tenant_id': 'test-tenant',
-                            'id': 'sg_fake_uuid', 'name': 'test_sg'}),
-                 mock.call(mock.ANY, None, 'security_group_rule',
-                           'sg_rule_fake_uuid', 'delete', ['sg_fake_uuid'])])
+                            'id': SG_FAKE_ID, 'name': 'test_sg'})])
 
     def test_sync_multiple_updates(self):
         # add 2 updates
@@ -634,6 +644,25 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
         rows = db.get_all_db_rows_by_state(self.db_session, 'pending')
         self.assertEqual(2, len(rows))
 
+    def test_update_port_filter(self):
+        """Validate the filter code on update port operation"""
+        expected_items = ['fixed_ips', 'security_groups', 'device_id',
+                          'security_groups', 'admin_state_up']
+        subnet = self._get_mock_operation_context(odl_const.ODL_SUBNET).current
+        port = self._get_mock_operation_context(odl_const.ODL_PORT).current
+        port['fixed_ips'] = [{'subnet_id': subnet['id'],
+                              'ip_address': '10.0.0.10'}]
+        port['mac_address'] = port['mac_address'].upper()
+
+        orig_port = copy.deepcopy(port)
+
+        with mock.patch.object(segments_db, 'get_network_segments'):
+            filters.filter_for_odl(odl_const.ODL_PORT,
+                                   odl_const.ODL_UPDATE, port)
+            for key, value in orig_port.items():
+                if key in expected_items:
+                    self.assertEqual(port[key], value)
+
 
 class _OpenDaylightDriverVlanTransparencyBase(base_v2.OpenDaylightTestCase):
     def _driver_context(self, network):
@@ -653,11 +682,11 @@ class TestOpenDaylightDriverVlanTransparencyNetwork(
         self.assertTrue(self.mech.check_vlan_transparency(context))
 
     def test_vlan_transparency(self):
-        for network_type in [p_const.TYPE_VXLAN]:
+        for network_type in [n_constants.TYPE_VXLAN]:
             self._test_network_type(True, network_type)
-        for network_type in [p_const.TYPE_FLAT, p_const.TYPE_GENEVE,
-                             p_const.TYPE_GRE, p_const.TYPE_LOCAL,
-                             p_const.TYPE_VLAN]:
+        for network_type in [n_constants.TYPE_FLAT, n_constants.TYPE_GENEVE,
+                             n_constants.TYPE_GRE, n_constants.TYPE_LOCAL,
+                             n_constants.TYPE_VLAN]:
             self._test_network_type(False, network_type)
 
 
@@ -666,19 +695,19 @@ class TestOpenDaylightDriverVlanTransparency(
     scenarios = [
         ("vxlan_vxlan",
          {'expected': True,
-          'network_types': [p_const.TYPE_VXLAN, p_const.TYPE_VXLAN]}),
+          'network_types': [n_constants.TYPE_VXLAN, n_constants.TYPE_VXLAN]}),
         ("gre_vxlan",
          {'expected': False,
-          'network_types': [p_const.TYPE_GRE, p_const.TYPE_VXLAN]}),
+          'network_types': [n_constants.TYPE_GRE, n_constants.TYPE_VXLAN]}),
         ("vxlan_vlan",
          {'expected': False,
-          'network_types': [p_const.TYPE_VXLAN, p_const.TYPE_VLAN]}),
+          'network_types': [n_constants.TYPE_VXLAN, n_constants.TYPE_VLAN]}),
         ("vxlan_flat",
          {'expected': False,
-          'network_types': [p_const.TYPE_VXLAN, p_const.TYPE_FLAT]}),
+          'network_types': [n_constants.TYPE_VXLAN, n_constants.TYPE_FLAT]}),
         ("vlan_vlan",
          {'expected': False,
-          'network_types': [p_const.TYPE_VLAN, p_const.TYPE_VLAN]}),
+          'network_types': [n_constants.TYPE_VLAN, n_constants.TYPE_VLAN]}),
     ]
 
     def test_network_segments(self):
