@@ -14,28 +14,84 @@
 #    under the License.
 
 from copy import deepcopy
-import mock
 from os import path as os_path
 from string import Template
 
-from neutron.plugins.common import constants
-from neutron.plugins.ml2 import driver_api as api
-from neutron.plugins.ml2 import driver_context as ctx
-from neutron_lib.api.definitions import portbindings
-from neutron_lib import constants as n_const
-from neutron_lib.plugins import directory
-
-from networking_odl.ml2 import pseudo_agentdb_binding
-from networking_odl.tests import base
+import fixtures
+import mock
+from oslo_config import cfg
+from oslo_serialization import jsonutils
 from requests.exceptions import HTTPError
 
+from neutron.db import provisioning_blocks
+from neutron.plugins.ml2 import driver_api as api
+from neutron.plugins.ml2 import driver_context as ctx
+from neutron.plugins.ml2 import plugin as ml2_plugin
 from neutron.tests.unit.db import test_db_base_plugin_v2 as test_plugin
+from neutron.tests.unit import testlib_api
+from neutron_lib.api.definitions import portbindings
+from neutron_lib import constants as n_const
+from neutron_lib.plugins import constants as plugin_constants
+from neutron_lib.plugins import directory
+from neutron_lib.plugins.ml2 import api as ml2_api
+
+from networking_odl.common import odl_features
+from networking_odl.common import websocket_client
+from networking_odl.journal import periodic_task
+from networking_odl.ml2 import pseudo_agentdb_binding
+from networking_odl.tests import base
+
 
 AGENTDB_BINARY = 'neutron-odlagent-portbinding'
 L2_TYPE = "ODL L2"
 
 
-class TestPseudoAgentDBBindingController(base.DietTestCase):
+class OpenDaylightAgentDBFixture(fixtures.Fixture):
+    def _setUp(self):
+        super(OpenDaylightAgentDBFixture, self)._setUp()
+        fake_agents_db = mock.MagicMock()
+        fake_agents_db.create_or_update_agent = mock.MagicMock()
+        directory.add_plugin(plugin_constants.CORE, fake_agents_db)
+
+
+class TestPseudoAgentDBBindingTaskBase(base.DietTestCase):
+    """Test class for AgentDBPortBindingTaskBase."""
+
+    def setUp(self):
+        """Setup test."""
+        self.useFixture(base.OpenDaylightRestClientFixture())
+        self.useFixture(OpenDaylightAgentDBFixture())
+        super(TestPseudoAgentDBBindingTaskBase, self).setUp()
+
+        self.worker = pseudo_agentdb_binding.PseudoAgentDBBindingWorker()
+        self.task = pseudo_agentdb_binding.PseudoAgentDBBindingTaskBase(
+            self.worker)
+
+    def _get_raised_response(self, json_data, status_code):
+
+        class MockHTTPError(HTTPError):
+            def __init__(self, json_data, status_code):
+                self.json_data = json_data
+                self.status_code = status_code
+                self.response = self
+
+        class MockResponse(object):
+            def __init__(self, json_data, status_code):
+                self.raise_obj = MockHTTPError(json_data, status_code)
+
+            def raise_for_status(self):
+                raise self.raise_obj
+
+        return MockResponse(json_data, status_code)
+
+    def test_hostconfig_response_404(self):
+        with mock.patch.object(self.task.odl_rest_client,
+                               'get', return_value=self.
+                               _get_raised_response({}, 404)):
+                self.assertEqual(self.task._rest_get_hostconfigs(), [])
+
+
+class TestPseudoAgentDBBindingWorker(base.DietTestCase):
     """Test class for AgentDBPortBinding."""
 
     # test data hostconfig and hostconfig-dbget
@@ -50,6 +106,24 @@ class TestPseudoAgentDBBindingController(base.DietTestCase):
                     "bridge_mappings": {"physnet1": "br-ex"}}"""}
     ]}}
 
+    def setUp(self):
+        """Setup test."""
+        self.useFixture(base.OpenDaylightRestClientFixture())
+        self.useFixture(OpenDaylightAgentDBFixture())
+        super(TestPseudoAgentDBBindingWorker, self).setUp()
+
+        self.worker = pseudo_agentdb_binding.PseudoAgentDBBindingWorker()
+
+    def test_update_agents_db(self):
+        """test agent update."""
+        self.worker.update_agents_db(
+            hostconfigs=self.sample_odl_hconfigs['hostconfigs']['hostconfig'])
+        self.worker.agents_db.create_or_update_agent.assert_called_once()
+
+
+class TestPseudoAgentDBBindingController(base.DietTestCase):
+    """Test class for AgentDBPortBinding."""
+
     # Test data for string interpolation of substitutable identifers
     #   e.g. $PORT_ID identifier in the configurations JSON string  below shall
     # be substituted with portcontext.current['id'] eliminating the check
@@ -164,84 +238,91 @@ class TestPseudoAgentDBBindingController(base.DietTestCase):
         "allowed_network_types": ["local", "vlan", "vxlan", "gre"],
         "bridge_mappings": {"physnet1": "br-ex"}}}
 
+    # Test data for length of string
+    sample_odl_hconfigs_length = {
+        "host": "devstack",      # host-id in ODL JSON
+        "agent_type": "ODL L2",  # host-type in ODL JSON
+                                 # config in ODL JSON
+        "configurations": {"supported_vnic_types": [
+            {"vnic_type": "normal", "vif_type": "vhostuser",
+             "vif_details": {
+                 "uuid": "TEST_UUID",
+                 "has_datapath_type_netdev": True,
+                 "support_vhost_user": True,
+                 "port_prefix": "longprefix_",
+                 "vhostuser_socket_dir": "/tmp",
+                 "vhostuser_ovs_plug": True,
+                 "vhostuser_mode": "server",
+                 "vhostuser_socket":
+                     "/tmp/longprefix_$PORT_ID"
+             }}],
+            "allowed_network_types": [
+                "local", "vlan", "vxlan", "gre"],
+            "bridge_mappings": {"physnet1": "br-ex"}}
+    }
+
+    # Raw test data for unicode/string comparison
+    sample_odl_hconfigs_length_raw = {
+        "host": "devstack",
+        "agent_type": "ODL L2",
+        "configurations": """{"supported_vnic_types": [
+            {"vnic_type": "normal", "vif_type": "vhostuser",
+             "vif_details": {
+                 "uuid": "TEST_UUID",
+                 "has_datapath_type_netdev": true,
+                 "support_vhost_user": true,
+                 "port_prefix": "prefix_",
+                 "vhostuser_socket_dir": "/tmp",
+                 "vhostuser_ovs_plug": true,
+                 "vhostuser_mode": "server",
+                 "vhostuser_socket":
+                     "/tmp/prefix_$PORT_ID"
+             }}],
+            "allowed_network_types": [
+                "local", "vlan", "vxlan", "gre"],
+            "bridge_mappings": {"physnet1": "br-ex"}}"""
+    }
+
     # test data valid  and invalid segments
     test_valid_segment = {
-        api.ID: 'API_ID',
-        api.NETWORK_TYPE: constants.TYPE_LOCAL,
-        api.SEGMENTATION_ID: 'API_SEGMENTATION_ID',
-        api.PHYSICAL_NETWORK: 'API_PHYSICAL_NETWORK'}
+        ml2_api.ID: 'API_ID',
+        ml2_api.NETWORK_TYPE: n_const.TYPE_LOCAL,
+        ml2_api.SEGMENTATION_ID: 'API_SEGMENTATION_ID',
+        ml2_api.PHYSICAL_NETWORK: 'API_PHYSICAL_NETWORK'}
 
     test_invalid_segment = {
-        api.ID: 'API_ID',
-        api.NETWORK_TYPE: constants.TYPE_NONE,
-        api.SEGMENTATION_ID: 'API_SEGMENTATION_ID',
-        api.PHYSICAL_NETWORK: 'API_PHYSICAL_NETWORK'}
+        ml2_api.ID: 'API_ID',
+        ml2_api.NETWORK_TYPE: n_const.TYPE_NONE,
+        ml2_api.SEGMENTATION_ID: 'API_SEGMENTATION_ID',
+        ml2_api.PHYSICAL_NETWORK: 'API_PHYSICAL_NETWORK'}
 
     def setUp(self):
         """Setup test."""
-        super(TestPseudoAgentDBBindingController, self).setUp()
         self.useFixture(base.OpenDaylightRestClientFixture())
+        self.useFixture(base.OpenDaylightFeaturesFixture())
+        self.useFixture(OpenDaylightAgentDBFixture())
+        super(TestPseudoAgentDBBindingController, self).setUp()
 
-        fake_agents_db = mock.MagicMock()
-        fake_agents_db.create_or_update_agent = mock.MagicMock()
-
-        self.mgr = pseudo_agentdb_binding.PseudoAgentDBBindingController(
-            db_plugin=fake_agents_db)
-
-    def test_make_hostconf_uri(self):
-        """test make uri."""
-        test_path = '/restconf/neutron:neutron/hostconfigs'
-        expected = "http://localhost:8080/restconf/neutron:neutron/hostconfigs"
-        test_uri = self.mgr._make_hostconf_uri(path=test_path)
-
-        self.assertEqual(expected, test_uri)
-
-    def test_update_agents_db(self):
-        """test agent update."""
-        self.mgr._update_agents_db(
-            hostconfigs=self.sample_odl_hconfigs['hostconfigs']['hostconfig'])
-        self.mgr.agents_db.create_or_update_agent.assert_called_once()
-
-    def _get_raised_response(self, json_data, status_code):
-
-        class MockHTTPError(HTTPError):
-            def __init__(self, json_data, status_code):
-                self.json_data = json_data
-                self.status_code = status_code
-                self.response = self
-
-        class MockResponse(object):
-            def __init__(self, json_data, status_code):
-                self.raise_obj = MockHTTPError(json_data, status_code)
-
-            def raise_for_status(self):
-                raise self.raise_obj
-
-        return MockResponse(json_data, status_code)
-
-    def test_hostconfig_response_404(self):
-        with mock.patch.object(self.mgr.odl_rest_client,
-                               'get', return_value=self.
-                               _get_raised_response({}, 404)):
-                self.assertEqual(self.mgr._rest_get_hostconfigs(), [])
+        self.mgr = pseudo_agentdb_binding.PseudoAgentDBBindingController()
 
     def test_is_valid_segment(self):
         """Validate the _check_segment method."""
-        all_network_types = [constants.TYPE_FLAT, constants.TYPE_GRE,
-                             constants.TYPE_LOCAL, constants.TYPE_VXLAN,
-                             constants.TYPE_VLAN, constants.TYPE_NONE]
+        all_network_types = [n_const.TYPE_FLAT, n_const.TYPE_GRE,
+                             n_const.TYPE_LOCAL, n_const.TYPE_VXLAN,
+                             n_const.TYPE_VLAN, n_const.TYPE_NONE]
 
         valid_types = {
             network_type
             for network_type in all_network_types
-            if self.mgr._is_valid_segment({api.NETWORK_TYPE: network_type}, {
-                'allowed_network_types': [
-                    constants.TYPE_LOCAL, constants.TYPE_GRE,
-                    constants.TYPE_VXLAN, constants.TYPE_VLAN]})}
+            if self.mgr._is_valid_segment(
+                {ml2_api.NETWORK_TYPE: network_type},
+                {'allowed_network_types': [
+                    n_const.TYPE_LOCAL, n_const.TYPE_GRE,
+                    n_const.TYPE_VXLAN, n_const.TYPE_VLAN]})}
 
         self.assertEqual({
-            constants.TYPE_LOCAL, constants.TYPE_GRE, constants.TYPE_VXLAN,
-            constants.TYPE_VLAN}, valid_types)
+            n_const.TYPE_LOCAL, n_const.TYPE_GRE, n_const.TYPE_VXLAN,
+            n_const.TYPE_VLAN}, valid_types)
 
     def test_bind_port_with_vif_type_ovs(self):
         """test bind_port with vanilla ovs."""
@@ -255,7 +336,7 @@ class TestPseudoAgentDBBindingController(base.DietTestCase):
             port_context, self.sample_hconfig_dbget_ovs)
 
         port_context.set_binding.assert_called_once_with(
-            self.test_valid_segment[api.ID], vif_type,
+            self.test_valid_segment[ml2_api.ID], vif_type,
             vif_details, status=n_const.PORT_STATUS_ACTIVE)
 
     def _set_pass_vif_details(self, port_context, vif_details):
@@ -283,7 +364,7 @@ class TestPseudoAgentDBBindingController(base.DietTestCase):
         self._set_pass_vif_details(port_context, pass_vif_details)
 
         port_context.set_binding.assert_called_once_with(
-            self.test_valid_segment[api.ID], pass_vif_type,
+            self.test_valid_segment[ml2_api.ID], pass_vif_type,
             pass_vif_details, status=n_const.PORT_STATUS_ACTIVE)
 
     def test_bind_port_with_vif_type_vhost_user_vpp(self):
@@ -300,7 +381,7 @@ class TestPseudoAgentDBBindingController(base.DietTestCase):
         self._set_pass_vif_details(port_context, pass_vif_details)
 
         port_context.set_binding.assert_called_once_with(
-            self.test_valid_segment[api.ID], pass_vif_type,
+            self.test_valid_segment[ml2_api.ID], pass_vif_type,
             pass_vif_details, status=n_const.PORT_STATUS_ACTIVE)
 
     def test_bind_port_without_valid_segment(self):
@@ -347,14 +428,80 @@ class TestPseudoAgentDBBindingController(base.DietTestCase):
 
         self.assertEqual(expected_str, test_string)
 
+    def test_str_template_substitution_length_in_configuration_string(self):
+        """Test for identifier substitution in config JSON string."""
+        port_context = self._fake_port_context(
+            fake_segments=[self.test_invalid_segment, self.test_valid_segment])
+
+        hconf_dict = self.mgr._substitute_hconfig_tmpl(
+            port_context, self.sample_odl_hconfigs_length)
+
+        test_string = hconf_dict['configurations'][
+            'supported_vnic_types'][0][
+            'vif_details'][portbindings.VHOST_USER_SOCKET]
+
+        expected_str = Template('/tmp/longprefix_$PORT_ID')
+        expected_str = expected_str.safe_substitute({
+            'PORT_ID': port_context.current['id']})
+
+        self.assertNotEqual(expected_str, test_string)
+        self.assertEqual(len(test_string) - len('/tmp/'), 14)
+
+    def test_template_substitution_in_raw_configuration(self):
+        """Test for identifier substitution in config string."""
+        port_context = self._fake_port_context(
+            fake_segments=[self.test_invalid_segment, self.test_valid_segment])
+
+        # Substitute raw string configuration with json
+        raw_configurations = self.sample_odl_hconfigs_length_raw[
+            'configurations']
+        raw_configurations_json = jsonutils.loads(raw_configurations)
+        self.sample_odl_hconfigs_length_raw['configurations'] = (
+            raw_configurations_json)
+
+        hconf_dict = self.mgr._substitute_hconfig_tmpl(
+            port_context, self.sample_odl_hconfigs_length_raw)
+
+        test_string = hconf_dict['configurations'][
+            'supported_vnic_types'][0][
+                'vif_details'][portbindings.VHOST_USER_SOCKET]
+
+        expected_str = Template('/tmp/prefix_$PORT_ID')
+        expected_str = expected_str.safe_substitute({
+            'PORT_ID': port_context.current['id']})
+
+        self.assertEqual(expected_str, test_string)
+
     def _fake_port_context(self, fake_segments, host_agents=None):
         network = mock.MagicMock(spec=api.NetworkContext)
         return mock.MagicMock(
             spec=ctx.PortContext,
-            current={'id': 'CONTEXT_ID',
+            current={'id': 'PORTID',
                      portbindings.VNIC_TYPE: portbindings.VNIC_NORMAL},
             segments_to_bind=fake_segments, network=network,
-            host_agents=lambda agent_type: host_agents)
+            host_agents=lambda agent_type: host_agents,
+            _plugin_context=mock.MagicMock()
+        )
+
+    @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
+    def test_prepare_inital_port_status_no_websocket(
+            self, mocked_add_provisioning_component):
+        odl_features.feature_set = set()
+        port_ctx = self._fake_port_context(
+            fake_segments=[self.test_valid_segment])
+        initial_port_status = self.mgr._prepare_initial_port_status(port_ctx)
+        self.assertEqual(initial_port_status, n_const.PORT_STATUS_ACTIVE)
+        mocked_add_provisioning_component.assert_not_called()
+
+    @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
+    def test_prepare_inital_port_status_with_websocket(
+            self, mocked_add_provisioning_component):
+        odl_features.feature_set.add(odl_features.OPERATIONAL_PORT_STATUS)
+        port_ctx = self._fake_port_context(
+            fake_segments=[self.test_valid_segment])
+        initial_port_status = self.mgr._prepare_initial_port_status(port_ctx)
+        self.assertEqual(initial_port_status, n_const.PORT_STATUS_DOWN)
+        mocked_add_provisioning_component.assert_called()
 
 
 class TestPseudoAgentDBBindingControllerBug1608659(
@@ -379,16 +526,60 @@ class TestPseudoAgentDBBindingControllerBug1608659(
     ]}}
 
     def setUp(self):
+        self.useFixture(base.OpenDaylightRestClientFixture())
+        self.useFixture(OpenDaylightAgentDBFixture())
         super(TestPseudoAgentDBBindingControllerBug1608659, self).setUp(
             plugin='ml2')
-        self.useFixture(base.OpenDaylightRestClientFixture())
-        self.core_plugin = directory.get_plugin()
-        self.mgr = pseudo_agentdb_binding.PseudoAgentDBBindingController(
-            self.core_plugin)
+        self.worker = pseudo_agentdb_binding.PseudoAgentDBBindingWorker()
 
     def test_execute_no_exception(self):
         with mock.patch.object(pseudo_agentdb_binding, 'LOG') as mock_log:
-            self.mgr._update_agents_db(
+            self.worker.update_agents_db(
                 self.sample_odl_hconfigs['hostconfigs']['hostconfig'])
             # Assert no exception happened
             self.assertFalse(mock_log.exception.called)
+
+
+class TestPseudoAgentNeutronWorker(testlib_api.SqlTestCase):
+    def setUp(self):
+        self.useFixture(base.OpenDaylightRestClientFixture())
+        self.useFixture(base.OpenDaylightJournalThreadFixture())
+        self.useFixture(base.OpenDaylightFeaturesFixture())
+        self.mock_periodic_thread = mock.patch.object(
+            periodic_task.PeriodicTask, 'start').start()
+        super(TestPseudoAgentNeutronWorker, self).setUp()
+        cfg.CONF.set_override('mechanism_drivers', ['opendaylight_v2'], 'ml2')
+        cfg.CONF.set_override('port_binding_controller',
+                              'pseudo-agentdb-binding', 'ml2_odl')
+
+    def test_get_worker(self):
+        workers = ml2_plugin.Ml2Plugin().get_workers()
+        self.assertTrue(any(
+            isinstance(worker,
+                       pseudo_agentdb_binding.PseudoAgentDBBindingWorker)
+            for worker in workers))
+
+    def test_worker(self):
+        worker = pseudo_agentdb_binding.PseudoAgentDBBindingWorker()
+        worker.wait()
+        worker.stop()
+        worker.reset()
+
+    def test_worker_start_websocket(self):
+        cfg.CONF.set_override('enable_websocket_pseudo_agentdb',
+                              True, 'ml2_odl')
+        worker = pseudo_agentdb_binding.PseudoAgentDBBindingWorker()
+        with mock.patch.object(
+                websocket_client.OpenDaylightWebsocketClient,
+                'odl_create_websocket') as mock_odl_create_websocket:
+            worker.start()
+            mock_odl_create_websocket.assert_called_once()
+
+    def test_worker_start_periodic(self):
+        cfg.CONF.set_override('enable_websocket_pseudo_agentdb',
+                              False, 'ml2_odl')
+        worker = pseudo_agentdb_binding.PseudoAgentDBBindingWorker()
+        with mock.patch.object(
+                periodic_task.PeriodicTask, 'start') as mock_start:
+            worker.start()
+            mock_start.assert_called_once()
