@@ -26,9 +26,9 @@ if [[ "$IS_GATE" != "True" ]] && [[ "$#" -lt 1 ]]; then
     >&2 echo "Usage: $0 /path/to/devstack [-i]
 Configure a host to run Networking ODL's functional test suite.
 
--i   Install Networking ODL's package dependencies.  By default, it is assumed
-     that devstack has already been used to deploy Networking ODL to the
-     target host and that package dependencies need not be installed.
+-i  Install Networking ODL's package dependencies.  By default, it is assumed
+    that devstack has already been used to deploy Networking ODL to the
+    target host and that package dependencies need not be installed.
 
 Warning: This script relies on devstack to perform extensive
 modification to the underlying host.  It is recommended that it be
@@ -60,8 +60,8 @@ INSTALL_BASE_DEPENDENCIES=${INSTALL_BASE_DEPENDENCIES:-$IS_GATE}
 
 
 if [ ! -f "$DEVSTACK_PATH/stack.sh" ]; then
-  >&2 echo "Unable to find devstack at '$DEVSTACK_PATH'.  Please verify that the specified path points to a valid devstack repo."
-  exit 1
+    >&2 echo "Unable to find devstack at '$DEVSTACK_PATH'.  Please verify that the specified path points to a valid devstack repo."
+    exit 1
 fi
 
 
@@ -74,6 +74,8 @@ function _init {
     FILES=$DEVSTACK_PATH/files
     TOP_DIR=$DEVSTACK_PATH
 
+    source $DEVSTACK_PATH/inc/meta-config
+    extract_localrc_section $TOP_DIR/local.conf $TOP_DIR/localrc $TOP_DIR/.localrc.auto
     source $DEVSTACK_PATH/stackrc
 
     # Allow the gate to override values set by stackrc.
@@ -186,9 +188,8 @@ function _install_opendaylight {
     # openstack service provider isn't needed, only ODL neutron northbound
     # is necessary for functional test
     ODL_NETVIRT_KARAF_FEATURE=odl-neutron-service,odl-restconf-all,odl-aaa-authn,odl-dlux-core,odl-mdsal-apidocs,odl-neutron-logger
-    if [[ "$VENV" =~ "fullstack" ]]; then
-        # TODO: in future switch to new netvirt
-        ODL_NETVIRT_KARAF_FEATURE=odl-neutron-service,odl-restconf-all,odl-aaa-authn,odl-dlux-core,odl-mdsal-apidocs,odl-ovsdb-openstack,odl-neutron-logger
+    if [[ "$VENV" =~ "dsvm-fullstack" ]]; then
+        export ODL_NETVIRT_KARAF_FEATURE=odl-neutron-service,odl-restconf-all,odl-aaa-authn,odl-dlux-core,odl-mdsal-apidocs,odl-netvirt-openstack,odl-neutron-logger
     fi
     ODL_BOOT_WAIT_URL=controller/nb/v2/neutron/networks
     source $NETWORKING_ODL_DIR/devstack/settings.odl
@@ -196,7 +197,7 @@ function _install_opendaylight {
     set +e
     curl -o /dev/null --fail --silent --head -u \
          ${ODL_USERNAME}:${ODL_PASSWORD} \
-         http://${ODL_MGR_IP}:${ODL_PORT}/${ODL_BOOT_WAIT_URL}
+         http://${ODL_MGR_HOST}:${ODL_PORT}/${ODL_BOOT_WAIT_URL}
     local result=$?
     set -e
     if [ $result -eq 0 ]; then
